using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Cathexis.IDBlue;

namespace SDPluginRFID
{

    #region Public Delegates releated to Service Discovery

    // Delegates used to call ServiceGUI functions from discovery worker thread
    public delegate void DelegateUpdateConnectionState(bool b);

    #endregion



    public partial class SDPluginGUI : Form
    {
        
        //The worker thread that will provide Service Discovery connectivity
        private Thread m_WorkerThread;

        //Events used to stop discovery worker thread
        private ManualResetEvent m_EventStopThread;
        private ManualResetEvent m_EventThreadStopped;

        // Delegate instance used to call GUI functions from discovery thread)
        public DelegateUpdateConnectionState m_DelegateUpdateConnectionState;

        //The service discovery thread
        private ServiceDiscoveryThread sdThread;

        // IDBLue .NET driver object
        private static IDBlueNET m_driver = null;

        // True if an idblue is connected, false othervise
        private bool isConnected = false;

        //A list containing the discovered Device objects
        private List<Device> devicesFoundList = new List<Device>();

        //A list containing the stored Device objects
        private List<Device> devicesPersistantList = new List<Device>();

        //The number of blocks to read if plugin is in blockread mode
        private static int BLOCKREAD_BLOCKNUMBER = 12;

        //Default host for Service Discovery Manager
        private static string defaultSDHost = "http://localhost:8080";
        


        /// <summary>
        /// This constructor will initialise the service discovery thread and
        /// the IDBlue RFID driver
        /// </summary>
        public SDPluginGUI()
        {
            InitializeComponent();

            txtUbiNode.Text = defaultSDHost;

            //initialize discovery delegate method
            m_DelegateUpdateConnectionState = new DelegateUpdateConnectionState(this.UpdateSDConnectionState);

            //Initialize service discovery thread events
            m_EventStopThread = new ManualResetEvent(false);
            m_EventThreadStopped = new ManualResetEvent(false);

            //Load the persistantly stored devices and update GUI
            devicesPersistantList = DeviceStore.LoadDevices();
            UpdateDeviceListBoxes();
            
            //Initialize the IDBlue driver class;
            m_driver = Driver;

            //Bind Driver events to local methods, for handling
            m_driver.OnDeviceActivated += new DeviceActivated(driver_OnDeviceActivated);
            m_driver.OnSearchComplete += new SearchComplete(driver_OnSearchComplete);
            m_driver.OnDeviceDeactivated += new DeviceDeactivated(driver_OnDeviceDeactivated);
            m_driver.OnAsyncTagID += new AsyncTagID(driver_OnAsyncTagID);
            m_driver.OnAsyncReadBlocks += new AsyncTagEvent(driver_OnAsyncReadBlocks);
        }

        

        /// <summary>
        /// When the form is closed, shut down the connection to the IDBlue,
        /// dispose of the driver and service discovery threads.
        /// </summary>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (m_driver.IsConnected)
                m_driver.Disconnect();
            m_driver.Dispose();
            StopServiceDiscoveryThread();
        }



        /// <summary>
        /// Property encapsulation of the IDBlue driver. If a driver does not exist one will 
        /// be instanciatedusing the Franson conection method.
        /// Singelton pattern employed to ensure only one driver exist at all times.
        /// </summary>
        public static IDBlueNET Driver
        {
            get
            {
                if (m_driver == null)
                    m_driver = new IDBlueNET(IDBlueConnectionMethod.Franson);
                return m_driver;
            }
        }



        #region Service Discovery Thread functions

        /// <summary>
        /// Starts the service discovery thread
        /// </summary>
        private void StartServiceDiscoveryThread()
        {
            //Reset events
            m_EventStopThread.Reset();
            m_EventThreadStopped.Reset();

            //Create worker thread instance
            m_WorkerThread = new Thread(new ThreadStart(this.serviceDiscoveryThreadFunction));
            m_WorkerThread.Name = "Service Discovery Thread";	// looks nice in Output window
            m_WorkerThread.Start();
        }



        /// <summary>
        /// Target for the StartServiceDiscoveryThread, to actually invoke the thread.
        /// </summary>
        private void serviceDiscoveryThreadFunction()
        {
            sdThread = new ServiceDiscoveryThread(m_EventStopThread, m_EventThreadStopped, 
                this, defaultSDHost);
            sdThread.Run();
        }



        /// <summary>
        /// Stop service discovery worker thread if it is running.
        /// </summary>
        private void StopServiceDiscoveryThread()
        {
            if (m_WorkerThread != null && m_WorkerThread.IsAlive)  // thread is active
            {
                m_EventStopThread.Set();

                //Wait to see if thread will stop/finish
                while (m_WorkerThread.IsAlive)
                {
                    // Infinite wait cannot be used because the SD thread makes syncronous
                    // calls to main form, this will cause deadlock. Instead, wait for
                    // event some appropriate time (and give time to SD thread) and
                    // process events. These events may contain Invoke calls.
                    if (WaitHandle.WaitAll(
                        (new ManualResetEvent[] { m_EventThreadStopped }),
                        100,
                        true))
                    {
                        break;
                    }
                    Application.DoEvents();
                }
            }
            UpdateSDConnectionState(false);
        }

        #endregion


        #region IDBlue event handlers

        /// <summary>
        /// This method is invoked when the IDBlue are set in Asyncron select tag id mode,
        /// and the IDBlue have read a tag.
        /// </summary>
        private void driver_OnAsyncTagID()
        {
            string tagid = Driver.AsyncData.TagId;
            Message("Read tag ID = " + tagid);
            sdThread.RegisterService(tagid, Driver.Device, "");
        }


        /// <summary>
        /// This method is invoked when the IDBlue are set in Asyncron read multiple blocks mode,
        /// and a tag with data blocks have been read by the IDBlue device.
        /// </summary>
        private void driver_OnAsyncReadBlocks()
        {
            string tagId = Driver.AsyncData.TagId;
            byte[] data = Driver.AsyncData.data.Data;

            //convert byte array to ascii string
            ASCIIEncoding enc = new ASCIIEncoding();
            string dataString = enc.GetString(data);
            string dataStr = dataString.Substring(0, dataString.IndexOf(@" ")).Trim();

            WebResolver wr = new WebResolver();
            List<Service> sList = wr.ResolveURL(dataStr);

            foreach (Service s in sList)
                sdThread.RegisterService(s);

            Message("Read tag ID = " + tagId + ", data = " + dataStr);
            //sdThread.RegisterService(tagId, Driver.Device, txtUbiNode.Text);
        }



        /// <summary>
        /// The search complete event is raised when the Bluetooth device 
        /// discovery has completed.
        /// </summary>
        private void driver_OnSearchComplete()
        {
            Message("Bluetooth Discovery complete. Please select the IDBlue to connect to.");
            btnStartDiscovery.Enabled = true;
            
            lbDiscoveredDevices.BeginUpdate();
            lbDiscoveredDevices.Items.Clear();

            devicesFoundList.Clear();
            devicesFoundList.AddRange(Driver.GetDevices());
            
            foreach (Device d in devicesFoundList)
            {
                lbDiscoveredDevices.Items.Add(d.Name + " (" + d.AddressStr + ")");
            }
            lbDiscoveredDevices.EndUpdate();
        }


        /// <summary>
        /// The device activated event is raised when a connection has been successfully 
        /// established with a remote Bluetooth device
        /// </summary>
        private void driver_OnDeviceActivated()
        {
            if (Driver.IsConnected)
            {
                //tabControl1.Enabled = true;
                isConnected = true;
                btnConnect.Text = "Disconnect";
                UpdateRFIDConnectionState("Connected to: " + Driver.Device);
                Message("Connected successfully to device " + Driver.Device);
                grpSelectMode.Enabled = true;
                setOperatingMode();
                StartServiceDiscoveryThread();
            }
        }

        
        /// <summary>
        /// This is the event handler for the device deactivated event.
        /// </summary>
        private void driver_OnDeviceDeactivated()
        {
            if (!Driver.IsConnected)
            {
                isConnected = false;
                btnConnect.Text = "Connect";
                
                grpSelectMode.Enabled = false;
                UpdateRFIDConnectionState("Not connected to RFID.");
                Message("Disconnected from device " + Driver.Device);
                StopServiceDiscoveryThread();
            }
        }


        private void setOperatingMode()
        {
            if (Driver.IsConnected)
            {
                if (radioTagmode.Checked == true)
                {
                    ErrorCode result = m_driver.BeginSelectTagId();
                    if (result != ErrorCode.Success)
                        Message("Error: " + result + " when trying to set \"Select Tag ID\" mode");
                    else
                        Message("IDBlue is now in \"Select Tag ID\" mode");
                }
                else
                {
                    ErrorCode result = m_driver.BeginReadBlocks(0, BLOCKREAD_BLOCKNUMBER);
                    if (result != ErrorCode.Success)
                        Message("Error: " + result + " when trying to set \"Read Multiple Blocks\" mode");
                    else
                        Message("IDBlue is now in \"Read Multiple Blocks\" mode");
                }
            }
        }


        #endregion

        

        #region GUI update functions

        /// <summary>
        /// This method is invoked by the Service Discovery Thread to update the
        /// state of the connection. This state is displayed in the 
        /// </summary>
        /// <param name="b">true if connected, false otherwise</param>
        private void UpdateSDConnectionState(bool b)
        {
            if (b)
                toolstripSD.Text = "Connected to Service Discovery";
            else
                toolstripSD.Text = "Not connected to Service Discovery";
        }
        

        /// <summary>
        /// Will write a string message to the message window.
        /// </summary>
        /// <param name="s">The string to output</param>
        private void Message(string s)
        {
            lbConsole.Items.Insert(0, s);
        }

        private void UpdateRFIDConnectionState(string s)
        {
            toolstripRFID.Text = s;
        }

        private void UpdateDeviceListBoxes()
        {
            lbDiscoveredDevices.Items.Clear();
            foreach (Device d in devicesFoundList)
                lbDiscoveredDevices.Items.Add(d.Name + " (" + d.AddressStr + ")");
            lbPersistantDevices.Items.Clear();
            foreach (Device d in devicesPersistantList)
                lbPersistantDevices.Items.Add(d.Name + " (" + d.AddressStr + ")");
        }

        #endregion


        /// <summary>
        /// Handler for the Start bluetooth discovery button click
        /// </summary>
        private void btnStartDiscovery_Click(object sender, EventArgs e)
        {
            Message("Initiating Bluetooth Discovery...");
            btnStartDiscovery.Enabled = false;
            Driver.StartDeviceSearch();
        }

        
        /// <summary>
        /// Button handler for the Add discovered device to the persistant devices
        /// button click.
        /// </summary>
        private void btnAddPersistantDevice_Click(object sender, EventArgs e)
        {
            if (lbDiscoveredDevices.SelectedIndex != -1)
            {
                int index = lbDiscoveredDevices.SelectedIndex;
                Device d = (Device)devicesFoundList[index];
                //check if new device exist in stored devices.
                bool exist = false;
                foreach (Device d2 in devicesPersistantList)
                    if (d2.AddressStr == d.AddressStr)
                        exist = true;
                if (exist)
                    MessageBox.Show("This device already exist in the used devices list.");
                else
                {
                    devicesFoundList.RemoveAt(index);
                    devicesPersistantList.Add(d);
                    DeviceStore.SaveDevices(devicesPersistantList);
                    UpdateDeviceListBoxes();
                }
            }
            else
                MessageBox.Show("You must select a discovered device before you can add it.");
        }



        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (m_driver.IsConnected)
            {
                Message("Disconnecting from device " + m_driver.Device);
                m_driver.Disconnect();
            }
            else
            {
                int index = lbPersistantDevices.SelectedIndex;
                if (index != -1)
                {
                    Device d = devicesPersistantList[index];
                    m_driver.Device = d.AddressStr;

                    // Initiate a manual connection
                    ErrorCode err = m_driver.BeginConnection();
                    if (err != ErrorCode.Success)
                    {
                        Message("Error: could not initiate connection to device");
                        return;
                    }
                }
                else
                    MessageBox.Show("You must select a device before you can connect to it.");
            }
        }


        /// <summary>
        /// Handler for 
        /// </summary>
        private void btnChangeMode_Click(object sender, EventArgs e)
        {
            setOperatingMode();
        }
        


    }
}