using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using SDPluginRFID.DiscoveryManager;

namespace SDPluginRFID
{
    /// <summary>
    /// This class provides a thread that will register a client with the Service 
    /// Domain Manager, and check if any services matching requests are found.
    /// </summary>
    public class ServiceDiscoveryThread
    {

        #region Class members

        /* If no web-service is specified, try to connect to this */
        private static string protocolName = "RFID";

        /* If no web-service is specified, try to connect to this */
        private static string defaultSDHost = "http://localhost:8080";
        
        /* URL to the Discovery Manager, with a default value */
        private string discoveryManagerURL = "/axis/services/discoveryManager";

        /* Main thread sets this event to stop worker thread */
        ManualResetEvent m_EventStop;

        /* Worker thread sets this event when it is stopped */
        ManualResetEvent m_EventStopped;

        /* Reference to main form, used to make syncronous user interface calls */
        private SDPluginGUI m_form;

        /* Web Reference to the Service Discovery Manager */
        private DiscoveryManagerImplService discoveryManager;

        /* The client subscriber id at the Service Discovery Manager */
        int pluginId;

        #endregion



        #region Class Functions

        /// <summary>
        /// Constructor which initializes the thread events and connection to Service
        /// Discovery Manager.
        /// </summary>
        public ServiceDiscoveryThread(ManualResetEvent eventStop,
                           ManualResetEvent eventStopped,
                           SDPluginGUI form, string hostname)
        {
            m_EventStop = eventStop;
            m_EventStopped = eventStopped;
            m_form = form;
            discoveryManager = new DiscoveryManagerImplService();
            if (hostname == null || hostname == "")
                discoveryManager.Url = defaultSDHost + discoveryManagerURL;
            else
                discoveryManager.Url = hostname + discoveryManagerURL;
        }


        /// <summary>
        /// The code within the Run method is the one which runs as a thread
        /// </summary>
        public void Run()
        {
            //counter will count to 10, then get a list of client requests and be reset.
            int counter = 0;

            //used to receive the xml-document with new client requests.
            string s;

            //Register this client with the Service Discovery Manager
            try
            {
                pluginId = discoveryManager.registerSDPlugin( protocolName );
                m_form.Invoke(m_form.m_DelegateUpdateConnectionState, new Object[] { true });
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(ex.StackTrace);
                pluginId = -1;
            }

            while (pluginId != -1)
            {
                counter++;
                Console.Out.WriteLine("PluginID=" + pluginId);

                //get services if any
                if ( counter == 10 )
                {
                    counter = 0;
                    try
                    {
                        s = discoveryManager.getClientRequest(pluginId);
                    }
                    catch (Exception ex)
                    {
                        Console.Out.WriteLine(ex.StackTrace);
                        pluginId = -1;
                    }
                }


                // check if thread is cancelled
                if (m_EventStop.WaitOne(0, true))
                {
                    m_EventStopped.Set(); // inform main thread that this thread stopped
                    return;
                }

                Thread.Sleep(500);
            }

            // Make asynchronous call to main form
            // to inform it that thread finished
            m_form.Invoke(m_form.m_DelegateUpdateConnectionState, new Object[] { false });
        }


        
        public void RegisterService(string tag, string address, string data)
        {
            try
            {
                discoveryManager.registerService(-1, pluginId, "", address, tag, "", data, "", "");
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(ex.StackTrace);
                pluginId = -1;
            }
        }


        public void RegisterService(Service s)
        {
            try
            {
                discoveryManager.registerService(-1, pluginId, s.DescriptionUri, s.ServiceUri,
                    s.Name, s.Type, s.Description, s.Owner, s.Location);
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(ex.StackTrace);
                pluginId = -1;
            }
        }


        #endregion



        #region Properties handlers

        /// <summary>
        /// This propery is the URL to the Discovery Manager Web server where the Web-service runs.
        /// This value will be used when connecting to it.
        /// </summary>
        public string DefaultSDHost
        {
            get { return defaultSDHost; }
            set { defaultSDHost = value; }
        }

        #endregion

    }

}
