/*
 * UbiCollab Service Discovery Plugin for Service Registry is developed by 
 * Kim-Steve Johansen as part of a master thesis at The Norwegian University 
 * of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.ArrayList;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.helpers.DefaultHandler;

/**
 * The Class ClientRequestParser implements a SAX <code>DefaultHandler</code>
 * which is costomized to parse a xml-document with client requests.
 * 
 * @author Kim-Steve Johansen
 * @version 0.6
 * @since 0.1
 */
public class ClientRequestParser extends DefaultHandler {

	
	/** The stack used to push and pop <code>ClientRequest</code> objects on/off. */
	private Stack<ClientRequest> stack;

    /** Controls if the stack is ready to accept text. */
    private boolean isStackReadyForText;

    /** The locator points to the xml-document location of the current SAX-event. */
    private Locator locator;
    
    /** The client request list. */
    private ArrayList<ClientRequest> clientRequestList;
	
	
    
    /**
     * This Constructor creates a new <code>ClientRequestParser</code> with
     * a stack and an empty client request list. The parser is immediately 
     * ready to handle parse events.
     */
    public ClientRequestParser() {
		stack = new Stack<ClientRequest>();
		clientRequestList = new ArrayList<ClientRequest>();
		isStackReadyForText = false;
	}
	
	
	
    /**
 	 * Gets the parsed client request list. Must only be called after the
 	 * parsing has been performed.
 	 * 
 	 * @return A typed <code>ArrayList</code> containing <code>ClientRequest</code> 
 	 *         objects, or a empty list if none found.
 	 */
 	public ArrayList<ClientRequest> getClientRequestList() { 
 		return clientRequestList; 
 	}
 	
    
 	
    /**
     * Inherited, but not used in this implementation. The application can use 
     * the <code>Locator</code> object to obtain the location of any other 
     * SAX event in the XML source document.
     * <p>
     * See <code>DefaultHandler#setDocumentLocator</code> for details.
     * 
     * @see org.xml.sax.helpers.DefaultHandler#setDocumentLocator(org.xml.sax.Locator)
     */
    public void setDocumentLocator( Locator rhs ) { 
    	if ( locator == null )
    		locator = rhs; 
    }
    
    
    
    /**
     * Invoked by the sax-documentreader when a <code>startElement</code> event
     * occurs.
     * <p>
     * Parameters are described in <code>DefaultHandler#startElement</code>
     * 
     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement( String uri, String localName, String qName, Attributes attribs ) {
		isStackReadyForText = false;
		if( localName.equals( "ClientRequest" ) ) {
		    stack.push( new ClientRequest() );
		    String attribValue;
		    //resolve ClientID
		    attribValue = resolveAttrib( uri, "ClientID", attribs, "%" );
		    ((ClientRequest)stack.peek()).setClientID( Integer.parseInt( attribValue ) );
		    //resolve name
		    attribValue = resolveAttrib( uri, "Name", attribs, "%" );
		    ((ClientRequest)stack.peek()).setName( attribValue );
		    //resolve Type
		    attribValue = resolveAttrib( uri, "Type", attribs, "%" );
		    ((ClientRequest)stack.peek()).setType( attribValue );
		    //resolve Location
		    attribValue = resolveAttrib( uri, "Location", attribs, "%" );
		    ((ClientRequest)stack.peek()).setLocation( attribValue );
		    //resolve Owner
		    attribValue = resolveAttrib( uri, "Owner", attribs, "%" );
		    ((ClientRequest)stack.peek()).setOwner( attribValue );
		    //resolve Description
		    attribValue = resolveAttrib( uri, "Description", attribs, "%" );
		    ((ClientRequest)stack.peek()).setDescription( attribValue );
		}		 
		else{
		    //System.out.println("SAX dicovered unknown element:" + localName);
			// do nothing (ClientRequestList will end up here)
		}		 
    }
    
    
    
    /**
     * Called by the sax-documentreader when a <code>endElement</code> event
     * occurs. 
     * <p>
     * Parameters are described in <code>DefaultHandler#endElement</code>
     * 
     * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement( String uri, String localName, String qName ) {
		isStackReadyForText = false;
		if( localName.equals( "ClientRequestList" ) ) {
		    while ( !stack.empty() )
		    	clientRequestList.add( stack.pop() );
		}
    }
    
    
    
    /**
     * Inherited, but not needed since the ClientRequestList xml structure
     * stores data purely in attributes. The purpose of this member is to
     * read element xml-values onto the stack.
     * <p>
     * See <code>DefaultHandler#characters</code> for more details.
     * 
     * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
     */
    public void characters( char[] data, int start, int length ) {
		if ( isStackReadyForText ) {
			//			 
		}
    }
    
    
    
    /**
     * Resolves the attribute given by <code>localName</code>, from the element 
     * being processed.
     * 
     * @param defaultValue the default value for the attribute
     * @param attr the list of xml attributes
     * @param uri the xml uri
     * @param localName the local name of the attribute
     * 
     * @return the value of the attribute if not null, <code>defaultValue</code>
     *         otherwise.
     */
    private String resolveAttrib( String uri, String localName, 
			          Attributes attr, String defaultValue ) {
		String tmp = attr.getValue( uri, localName );
		return (tmp!=null) ? (tmp) : (defaultValue);
    }
}

	
	
	

