/*
 * UbiCollab Service Discovery Plugin for Service Registry is developed by 
 * Kim-Steve Johansen as part of a master thesis at The Norwegian University 
 * of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.io.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;


/**
 * This Class is responsible for the logging of events, and for reading and returning
 * the log on request. The log file will be created in the folder specified in
 * <code>GlobalSettings.STORAGE_ROOT</code> on the system drive.
 * <p>
 * This class is created as a singleton class to enable logging from everywhere without
 * having to pass around a instance refrence. An instance might not be needed at all.
 * 
 * @author Kim-Steve Johansen
 * @version 0.5
 * @since 0.1
 */
public class Log {

	/** The logger. */
	private static Log logger=null;

	/** The directorypossible tells if the log directory is possible to write to. */
	private boolean directorypossible;

	/** The filename variable will be used to get the local system path to the log file. */
	private String filename;

	/** The Constant DEBUG describes the given log-level. */
	public final static int DEBUG = 0;

	/** The Constant INFO describes the given log-level. */
	public final static int INFO = 1;

	/** The Constant WARNING describes the given log-level. */
	public final static int WARNING = 2;

	/** The Constant ERROR describes the given log-level. */
	public final static int ERROR = 3;

	/** The Constant FATAL describes the given log-level. */
	public final static int FATAL = 4;
	
	/** The level of logging which is selected (e.g. DEBUG means logg everything). */
	private int logLevel;
	
	
	
	/**
	 * The only Constructor for the log, creates a new instance of this class.
	 * The reason why the visability is set to private, is because the log is a 
	 * singleton class. This implies that at most one instance of the class exist 
	 * at any time.
	 */
	private Log() {
		try {
			logLevel = GlobalSettings.SELECTED_LOG_LEVEL;
			String drive = java.io.File.listRoots()[0].getAbsolutePath();
			filename = drive + GlobalSettings.STORAGE_ROOT + System.getProperty("file.separator") + GlobalSettings.LOG_FILE;
			File path = new File(drive + GlobalSettings.STORAGE_ROOT);
			directorypossible = true;
			if ( !path.exists() ) {
				directorypossible = path.mkdirs();
			}
		}
		catch(Exception e){
			//System.out.println("Log file could not be created or opened, message:" + e.getMessage());
			System.out.println("Logging to sysout...");
			directorypossible = false;
		}
	}
	
	
	
	/**
	 * Method to get a refrence to the instance.
	 * 
	 * @return the instance
	 */
	public static Log getInstance(){
		if(logger==null){
			synchronized (Log.class) {
				if(logger==null){
					logger = new Log();
				}

			}
		}
		return logger;
	}



	/**
	 * Method to convert a date or time part, represented by a integer, 
	 * into a string representation with <code>length == 2</code>.  
	 * 
	 * @param item The item to convert
	 * 
	 * @return the <code>String</code> returned
	 */
	private String formatDT(int item) {
		if (item < 10)
			return "0"+item;
		else
			return ""+item;
	}
	
	
	
	/**
	 * Method to write a new record (og item) in the log. If the log file
	 * is unreachable, the <code>message</code> will be written to standard
	 * output.
	 * 
	 * @param level The log-level for the <code>message</code>.
	 * @param message The <code>String</code>g to write to the log
	 */
	public synchronized void writeToLog(String message, int level){
		if (directorypossible) {
			message = xmlEscape(message);
			try {
				BufferedWriter out = new BufferedWriter(new FileWriter(filename, true));
				Calendar cal = new GregorianCalendar();
				//Format date and time
				String time = formatDT(cal.get(Calendar.HOUR_OF_DAY)) + ":" + formatDT(cal.get(Calendar.MINUTE)) + ":" + formatDT(cal.get(Calendar.SECOND));
				String date = formatDT(cal.get(Calendar.DAY_OF_MONTH)) + ":" + formatDT(cal.get(Calendar.MONTH)) + ":" + formatDT(cal.get(Calendar.YEAR));
				
				//String time = ""+h+":"+m+":"+sec;
				out.write("\n");
				if(level>=logLevel){
					switch(level){
					case DEBUG:
						out.write("<Logitem Date=\"" + date + "\" Time=\""+time+"\" Level=\"DEBUG\" Message=\"" + message + "\" />");
						break;
					case INFO:
						out.write("<Logitem Date=\"" + date + "\" Time=\""+time+"\" Level=\"INFO\" Message=\"" + message + "\" />");
						break;
					case WARNING:
						out.write("<Logitem Date=\"" + date + "\" Time=\""+time+"\" Level=\"WARNING\" Message=\"" + message + "\" />");
						break;	
					case ERROR:
						out.write("<Logitem Date=\"" + date + "\" Time=\""+time+"\" Level=\"ERROR\" Message=\"" + message + "\" />");
						break;
					case FATAL:
						out.write("<Logitem Date=\"" + date + "\" Time=\""+time+"\" Level=\"FATAL\" Message=\"" + message + "\" />");
						break;
					default:
						out.write("<Logitem Date=\"" + date + "\" Time=\""+time+"\" Level=\"NO LEVEL\" Message=\"" + message + "\" />");
					}
				}
				out.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
				System.out.println(message);
			}
		}
		else {
			System.out.println("Logmessage: " + message);
		}
		
	}
	
	
	
	/**
	 * Read from the log and return selected log items in a XML-formatted string.
	 * <p>
	 * The xml-structure used to represent the returned log is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Log&gt;
	 *          &lt;Logitem Date=&quot;dd.mm.yyyy&quot; 
	 *            Time=&quot;hh:mm:ss&quot; 
	 *            Level=&quot;String&quot; 
	 *            Message=&quot;String&quot;/&gt;<br/> 
	 *          ... more log items ...<br/>
	 *      &lt;/Log&gt;
	 * </pre>
	 * 
	 * @param logLevel The log level to include in the report. Valid arguments are:
	 *      'DEBUG', 'INFO', 'WARNING', 'ERROR', 'FATAL' and 'ALL' for all types.
	 * @param numItems The number of log items to include in the report
	 * 
	 * @return the <code>String</code> containg the resulting XML
	 */
	public String readFromLog(String logLevel, int numItems) {
		String resultXML = GlobalSettings.XML_HEADER + "<Log>";
		String textLine;
		if (logLevel.equals("ALL")) logLevel = "Date";
		ArrayList<String> logItems = new ArrayList<String>();
		int lineCounter = 0;
		try {
			BufferedReader inFile = new BufferedReader(new FileReader(filename));
			while((textLine = inFile.readLine()) != null) {
				if (textLine.indexOf(logLevel) != -1) {
					lineCounter++;
					if (lineCounter > numItems) logItems.remove(0); 
					logItems.add(textLine);
				}
			}
			//Close the file
			inFile.close();
		}
		catch(IOException ioe) {
			ioe.printStackTrace();
		}
		while (logItems.size() > 0)
			resultXML += logItems.remove(0);
		return resultXML + "</Log>";
	}
	
	
	
	/**
     * Replace special characters with XML escapes. 
     * XML has the following five predeclared entities:
     * <pre>
     * &amp; <small>(ampersand)</small> is replaced by &amp;amp;
     * &lt; <small>(less than)</small> is replaced by &amp;lt;
     * &gt; <small>(greater than)</small> is replaced by &amp;gt;
     * &apos; <small>(apostrophe)</small> is replaced by &amp;apos;
     * &quot; <small>(double quote)</small> is replaced by &amp;quot;
     * </pre>
     * @param string The string to be escaped.
     * @return The escaped string.
     */
    private String xmlEscape(String string) {
        if (string == null)
        	string = "";
    	StringBuffer sb = new StringBuffer();
        for (int i = 0, len = string.length(); i < len; i++) {
            char c = string.charAt(i);
            switch (c) {
            case '&':
                sb.append("&amp;");
                break;
            case '<':
                sb.append("&lt;");
                break;
            case '>':
                sb.append("&gt;");
                break;
            case '\'':
                sb.append("&apos;");
                break;
            case '"':
                sb.append("&quot;");
                break;
            default:
                sb.append(c);
            }
        }
        return sb.toString();
    }
    

}
