/*
 * UbiCollab Service Discovery Plugin for Service Registry is developed by 
 * Kim-Steve Johansen as part of a master thesis at The Norwegian University 
 * of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.io.IOException;
import java.net.MalformedURLException;
import javax.xml.rpc.ServiceException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import no.ubicollab.axis.services.discoveryManager.*;
import no.ubicollab.axis.services.serviceRegistry.*;


public class PluginServiceRegistryImpl {

	/* The Service Registry Manager interface proxy. Need full package name. */
	private no.ubicollab.axis.services.serviceRegistry.ServiceRegistryImpl srProxy;
	
	/* The Service Discovery Manager interface proxy. Need full package name. */
	private no.ubicollab.axis.services.discoveryManager.DiscoveryManagerImpl dmProxy;
	
	/* The subscriber id at the Service Discovery Manager */
	private int sdID = -1;
	
	
	
	/**
	 * This Constructor creates Web-service proxies for Service Registry and
	 * Service Discovery Manager, and conects to Discovery Manager. In
	 * addition the <code>TimeKeeper</code> Thread is started.
	 */
	public PluginServiceRegistryImpl() {
		Log log = Log.getInstance();
		try {
			//create the Service Registry proxy 
			String srUrl = "http://localhost:8080/axis/services/serviceRegistry";
			ServiceRegistryImplServiceLocator srLocator = new ServiceRegistryImplServiceLocator();
			srProxy = srLocator.getserviceRegistry(new java.net.URL(srUrl));
			
			//create the Service Discovery proxy 
			String sdUrl = "http://localhost:8080/axis/services/discoveryManager";
			DiscoveryManagerImplServiceLocator sdLocator = new DiscoveryManagerImplServiceLocator();
			dmProxy = sdLocator.getdiscoveryManager(new java.net.URL(sdUrl));
			
			//Connect to Discovery Manager
			sdID = connectToDiscoveryManager();
			
			TimeKeeper thread = new TimeKeeper(this);
			thread.start();
			System.out.println("Service Registry Plugin Thread is started");
		} 
		catch (MalformedURLException e) {
			log.writeToLog("Invalid path (URL) to service specified. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		} 
		catch (ServiceException e) {
			log.writeToLog("Error while contacting Web-service. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		} 
		catch (Exception e) {
			log.writeToLog("The Web-service I am talking to is experiencing problems. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Connects to discovery manager and returns a subscriber id.
	 * 
	 * @return An <code>int</code> that is the subscriber id on the Service
	 *         Discovery Manager or -1 if unable to connect.
	 * 
	 * @throws RemoteException the remote exception received from the Service
	 *         Discovery Manager
	 */
	public int connectToDiscoveryManager() {
		Log log = Log.getInstance();
		try {
			return dmProxy.registerSDPlugin(GlobalSettings.PLUGIN_PROTOCOL);
		} catch (RemoteException e) {
			log.writeToLog("Unable to connect to Service Discovery. " +
					"Message:" + e.getMessage(), Log.WARNING);
			e.printStackTrace();
		}
		return -1;
	}
	
	
	
	/**
	 * This method is used to query the Service Domain Manager for any new
	 * Client Requests. The XML-response will be parsed and the 
	 * <code>ClientRequest</code> objects are returned.
	 * 
	 * @return An array of <code>ClientRequest</code> in a typed <code>ArrayList</code>
	 * 
	 * @throws RemoteException the remote exception received from the host.
	 */
	public ArrayList<ClientRequest> getClientRequests() {
		Log log = Log.getInstance();
		
		String s = "";
		try {
			s = dmProxy.getClientRequest( getSdID() );
		} catch (RemoteException e1) {
			setSdID(-1);
			log.writeToLog("Unable to retrieve client request list. " +
					"Message:" + e1.getMessage(), Log.ERROR);
			e1.printStackTrace();
		}
		
		//Prepare the parser
		InputSource src = new InputSource( parseStringToIS(s) );
		ClientRequestParser saxClientParser = new ClientRequestParser();
		XMLReader rdr;
		
		//Try to parse the received list
		try {
			rdr = XMLReaderFactory.createXMLReader();
			rdr.setContentHandler( saxClientParser );
			rdr.parse( src );
		} catch (SAXException e) {
			log.writeToLog("SAX parsing error encountered while parsing client request list. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		} catch (IOException e) {
			log.writeToLog("IO error encountered while parsing client request list. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		return saxClientParser.getClientRequestList();
	}
	
	
	
	/**
	 * Method to parse a provided <code>String</code> to a 
	 * <code>java.io.InputStream</code> object.
	 * 
	 * @param xml the xml to parse
	 * 
	 * @return a valid input stream reading from the xml string
	 */
	private java.io.InputStream parseStringToIS(String xml){
		if ( xml==null ) 
			return null; 
		xml = xml.trim();
		java.io.InputStream in = null;
		try{
			in = new java.io.ByteArrayInputStream(xml.getBytes("UTF-8"));
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
		return in; 
	}

	
	/**
	 * Method to parse a provided <code>String</code> to a 
	 * <code>java.io.InputStream</code> object.
	 * 
	 * @param xml the xml to parse
	 * 
	 * @return a valid input stream reading from the xml string
	 * 
	 */
	public ArrayList<Service> getServiceList(String name, String type, String location, String owner, String description) throws RemoteException {
		Log log = Log.getInstance();
		String s = "";
		
		s = srProxy.getServices(name, type, location, owner, description);
		
		//Prepare the parser
		InputSource src = new InputSource( parseStringToIS(s) );
		ServiceParser saxServiceParser = new ServiceParser();
		XMLReader rdr;
		
		//Try to parse the received list
		try {
			rdr = XMLReaderFactory.createXMLReader();
			rdr.setContentHandler( saxServiceParser );
			rdr.parse( src );
		} catch (SAXException e) {
			log.writeToLog("SAX parsing error encountered while parsing service list. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		} catch (IOException e) {
			log.writeToLog("IO error encountered while parsing service list. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		
		return saxServiceParser.getServiceList();
	}
	
	
	/**
	 * This method will register the provided service parameters with the
	 * Service Domain Manager.
	 * 
	 * @param friendlyName The user-friendly name of the service
	 * @param type The service type (classification)
	 * @param descriptionURI URI for service description (e.g. the URL to a WSDL-file).
	 * @param owner The person/company/responsible for the service.
	 * @param description Tags (words or short sentences) describing the service.
	 * @param location Location where the service can be found
	 * @param protocolID  The id for the service discovery protocol plugin.
	 * @param searchRequestorID the ID for the search request queue to add this service to
	 * @param serviceURI The URI to invoke the service
	 * 
	 * @return True if the service is successfully registered, false otherwise.
	 */
	public boolean registerService(int searchRequestorID, int protocolID, String descriptionURI,
			String serviceURI, String friendlyName, String type, String description, 
			String owner, String location) {
		Log log = Log.getInstance();
		boolean registeredOK = false;
		try {
			if ( dmProxy.registerService(searchRequestorID, protocolID, descriptionURI, serviceURI, friendlyName, type, description, owner, location) )
				registeredOK = true;
			else {
				//might have failed if the client have timed out
				sdID = connectToDiscoveryManager();
				registeredOK = dmProxy.registerService(searchRequestorID, protocolID, descriptionURI, serviceURI, friendlyName, type, description, owner, location);
			}
		} 
		catch (RemoteException e) {
			log.writeToLog("Remote exception encountered while registering service with Domain Manager. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		return registeredOK;
	}


	/**
	 * Gets the service discovery ID (subscriber id).
	 * 
	 * @return the service discovery ID
	 */
	public int getSdID() {
		return sdID;
	}


	/**
	 * Sets the service discovery ID (subscriber id).
	 * 
	 * @param the service discovery ID
	 */
	public void setSdID(int sdID) {
		this.sdID = sdID;
	}
	
	
	
	
}