/*
 * UbiCollab Service Discovery Plugin for Service Registry is developed by 
 * Kim-Steve Johansen as part of a master thesis at The Norwegian University 
 * of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.rmi.RemoteException;
import java.util.ArrayList;


/**
 * The Class <code>TimeKeeper</code> run as a seperate thread and is responsible 
 * for picking up new <code>ClientRequest</code> from Service Discovery Manager, 
 * and try to fulfill these.
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class TimeKeeper extends Thread {
	
	/** The running. */
	private static boolean running = true;
	
	/** Reference to the instance that started this thread. It further have
	 * methods for communicating with the other services. */
	private PluginServiceRegistryImpl srPlugin = null;
	
	/** The client request list received from Service Discovery Manager. */
	private ArrayList<ClientRequest> clientRequest;
	
	
	
	/**
	 * This Constructor creates the neccessary reference to the
	 * <code>PluginServiceRegistryImpl</code>.
	 * 
	 * @param srPlugin the instance that invoked this class.
	 * 
	 * @see PluginServiceRegistryImpl#PluginServiceRegistryImpl
	 */
	public TimeKeeper(PluginServiceRegistryImpl srPlugin) {
		this.srPlugin = srPlugin;
	}
	
	
	/**
	 * The code in this method starts to run as a thread when the inherited 
	 * <code>Thread#start</code> have been called. 
	 * 
	 * @see java.lang.Thread#run()
	 */
	public void run() {
		while (running) {
			
			//If connected to Service Discovery, try to retrieve a list of new 
			//Client Requests. If not, wait 15s and try to reconnect.
			if ( srPlugin.getSdID() != -1 )
				clientRequest = srPlugin.getClientRequests();
			else {
				try {
					Thread.sleep(15000);
					srPlugin.setSdID( srPlugin.connectToDiscoveryManager() );
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			//Handle all requests received
			while ( clientRequest.size() > 0 ) {
				clientRequestHandler( clientRequest.remove(0) );
			}
				
			//Wait 3s between each check for client requests
			try {
				Thread.sleep(3000);
			} catch (InterruptedException ie) {
				ie.printStackTrace();
			}
		}
	}

	
	
	/**
	 * This method is the client request handler. It will be called whenever a
	 * new clientrequest is received.
	 * 
	 * @param cr the <code>ClientRequest</code> that need to be attended to.
	 * 
	 * @see PluginServiceRegistryImpl#getServiceList(String, String, String, String, String)
	 * @see PluginServiceRegistryImpl#registerService(int, int, String, String, String, String, String, String, String)
	 */
	private void clientRequestHandler(ClientRequest cr) {
		Log log = Log.getInstance();
		ArrayList<Service> sList = new ArrayList<Service>();
		try {
			sList = srPlugin.getServiceList(cr.getName(), cr.getType(), cr.getLocation(), cr.getOwner(), cr.getDescription() );
		} catch (RemoteException e) {
			log.writeToLog("Remote error encountered while fetching service list. " +
					"Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		
		//Register any services found with the Service Discovery Manager
		while ( sList.size() > 0 ) {
			Service s = sList.remove(0);
			srPlugin.registerService(cr.getClientID(), srPlugin.getSdID(), 
					s.getDescriptionURI(), s.getServiceURI(), s.getName(), 
					s.getType(), s.getDescription(), s.getOwner(), s.getLocation());
		}
	}
	
	
	
	/**
	 * This method will stop the <code>TimeKeeper</code> thread.
	 */
	public static void stopThread() {
		running = false;
		System.out.println("Service Discovery Plugin for Service Registry has stopped");
	}
	
	
	
	/**
	 * This method will reset the running member of the 
	 * <code>HouseKeeperThread</code> thread. This must be
	 * done before the thread can be started again.
	 */
	public static void resetThread() {
		running = true;
		System.out.println("Service Discovery Plugin for Service Registry has stopped");
	}
	
}
