/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.*;
import org.osgi.framework.*;

import no.ubicollab.osgi.service.discoverymanager.*;


/**
 * The <code>Activator</code> class is responsible for starting and stopping the  
 * Service Discovery Manager. The class contains the <code>Activator#start</code>   
 * and <code>Activator#stop</code> members, which are invoked by the OSGi framwork 
 * to start and stop the bundle and its service.
 * 
 * @author Kim-Steve Johansen
 * @version 0.2
 * @since 0.1
 */
public class Activator implements BundleActivator {

	/** The reference to the discovery manager service. */
	private DiscoveryManagerImpl discoveryManager = null;
	
	
	
	/**
	 * The <code>start</code> memeber is called by the framework to start the bundle.
	 * This member instanciates the discovery manager service and registeres it along
	 * with its properties with the framework.
	 * 
	 * @param bc The refrence to the <code>BundleContext</code> as provided by the framwork.
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			Properties properties = new Properties();
			properties.put("Language", "English");
			properties.put("SOAP.service.name", GlobalSettings.SOAP_SERVICE_NAME);
			discoveryManager = new DiscoveryManagerImpl();
			HouseKeeperThread.resetThread();
			bc.registerService(ServiceDiscovery.class.getName(),
					discoveryManager, properties);
			log.writeToLog("Service Discovery Manager has started.", Log.INFO);
			System.out.println("Service Discovery Manager has started.");
		} catch (Exception e) {
			log.writeToLog("Service Discovery Manager was unable to start normally. Message:"
					+ e.getMessage(), Log.ERROR);
			System.out.println("Failed to start Service Discovery Manager. Read log for details.");
			throw new BundleException("Failed to start Service Discovery Manager", e);
		}
	}

	
	/**
	 * The <code>stop</code> memeber is called by the framework to stop the bundle.
	 * This member nullifies the discovery manager instance and stops the 
	 * <code>HouseKeeperThread</code>
	 * 
	 * @param bc The refrence to the <code>BundleContext</code> as provided by the framwork.
	 * 
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			HouseKeeperThread.stopThread();
			discoveryManager = null;
			log.writeToLog("Service Discovery Manager has stopped.", Log.INFO);
			System.out.println("Service Discovery Manager has stopped.");
		} catch (Exception e) {
			log.writeToLog("Service Discovery Manager was unable to stop normally. Message:"
					+ e.getMessage(), Log.ERROR);
			System.out.println("Failed to stop Service Discovery Manager. Read log for details.");
			throw new BundleException("Failed to stop Service Discovery Manager", e);
		}
	}

}