/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.ArrayList;

/**
 * The Class <code>Client</code> is used to represent a client which 
 * are registered with the Service Discovery Manager. 
 * 
 * @author Kim-Steve Johansen
 * @version 0.2
 * @since 0.1
 */
public class Client {

	/** The client id. */
	private int id;
	
	/** The clients time to live. */
	private int timeToLive;
	
	/** The clients queue of found services. */
	private ArrayList<Service> serviceQueue;
	
	
	
	/**
	 * The Constructor for the Client class.
	 * 
	 * @param id the unique id for identifying the client
	 */
	public Client(int id) {
		super();
		this.id = id;
		timeToLive = GlobalSettings.CLIENT_TIME_TO_LIVE;
		serviceQueue = new ArrayList<Service>();
	}

	
	/**
	 * Adds a service to the clients service queue.
	 * 
	 * @param s the service to add
	 * 
	 * @return true, if service is added, false othervise
	 */
	public boolean addService(Service s) {
		return serviceQueue.add(s);
	}
	
	
	/**
	 * Returns the service list in XML-fromat and clear the list at the same time.
	 * 
	 * @return the XML-formatted service list
	 */
	public String getServiceList() {
		Service s;
		if ( serviceQueue.size() == 0 )
			return "No matching services";
		else {
			String xml = GlobalSettings.XML_HEADER;
			xml += "<Servicelist>";
			while ( serviceQueue.size() > 0 ) {
				s = serviceQueue.remove(0);
				xml += s.getXML();
			}
			xml += "</Servicelist>";
			return xml;
		}
	}
	
	
	
	/**
	 * Reset the time to live to a full lifetime, as defined by 
	 * <code>GlobalSettings</code>.
	 */
	public void resetTimeToLive() {
		timeToLive = GlobalSettings.CLIENT_TIME_TO_LIVE;
	}
	
	
	
	/**
	 * Gets the unique id.
	 * 
	 * @return the id
	 */
	public int getId() {
		return id;
	}



	/**
	 * Sets the unique id.
	 * 
	 * @param id the id
	 */
	public void setId(int id) {
		this.id = id;
	}


	/**
	 * Gets the time to live remaining.
	 * 
	 * @return the time to live
	 */
	public int getTimeToLive() {
		return timeToLive;
	}


	/**
	 * Sets the time to live.
	 * 
	 * @param timeToLive the time to live
	 */
	public void setTimeToLive(int timeToLive) {
		this.timeToLive = timeToLive;
	}
	
}
