/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.ArrayList;

/**
 * The Class <code>ClientList</code> is used to represent all clients which
 * have registered with the Service Discovery Manager. 
 * <p>
 * The class provide the methods neccessary to handle <code>Client</code> related tasks.
 * 
 * @author Kim-Steve Johansen
 * @version 0.5
 * @since 0.1
 */
public class ClientList {

	/** The client counter. */
	private int clientCounter;
	
	/** The client list. */
	private ArrayList<Client> clientList;
	
	
	/**
	 * A Constructor which creates an instance with a empty client list 
	 * and a reset client counter.
	 */
	public ClientList() {
		clientCounter = 0;
		clientList = new ArrayList<Client>();
	}
	
	
	
	/**
	 * Creates a new client and returns the unique identifier for this client.
	 * 
	 * @return the int that is the unique id for the client.
	 */
	public int createNewClient() {
		clientCounter++;
		clientList.add(new Client(clientCounter));
		return clientCounter;
	}
	
	
	
	/**
	 * Gets the service queue for the given clientID in XML-format, and clears the
	 * queue at the same time.
	 * 
	 * @param clientID the client ID
	 * 
	 * @return the service list
	 */
	public String getServiceQueue(int clientID) {
		String queue = "No such ID";
		for (int i=0;i<clientList.size();i++)
			if ( clientList.get(i).getId() == clientID ) {
				queue = clientList.get(i).getServiceList(); 
				clientList.get(i).resetTimeToLive();
			}
		return queue;
	}
	
	
	
	/**
	 * Adds a service to the client queue for the client identified by clientID.
	 * Can also add a service to all listening clients by specifying
	 * <code>clientID = 1</code>.
	 * 
	 * @param clientID the client ID, or -1 for all clients
	 * @param s the service to add
	 * 
	 * @return true, if service is added, false othervise
	 */
	public boolean addServiceToQueue(int clientID, Service s) {
		boolean foundClient = false;
		for (int i=0;i<clientList.size();i++)
			if ( clientList.get(i).getId() == clientID )
				foundClient = clientList.get(i).addService(s);	
			else if ( clientID == -1 )
				foundClient = clientList.get(i).addService(s);
		return foundClient;
	}
	
	
	
	/**
	 * Decrease the time to live for all clients. Clients with time-to-live &lt;= 0 are
	 * assumed to have disconnected and will be removed.
	 * 
	 * @param time the time in millisecounds to decrease
	 */
	public synchronized void decreaseTimeToLive(int time) {
		int newTime;
		for (int i=0;i<clientList.size();i++) {
			newTime = clientList.get(i).getTimeToLive() - time;
			if ( newTime > 0 )
				clientList.get(i).setTimeToLive( newTime );
			else {
				Client c = clientList.remove(i);
				Log log = Log.getInstance();
				log.writeToLog("Client with id=" + c.getId() + " timed out and was removed.", 
						Log.DEBUG);
			}
		}
	}
	
}
