/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

/**
 * The Class ClientRequest is intended to store a single request for a
 * service, as registered by a client.
 * 
 * @author Kim-Steve Johansen
 * @version 0.5
 * @since 0.1
 */
public class ClientRequest {
	
	/** The client which registered this request. */
	private int clientID;
	
	/** The name to search for. */
	private String name;
	
	/** The type of service to search for. */
	private String type;
	
	/** The location of the service. */
	private String location;
	
	/** The owner of the service. */
	private String owner;
	
	/** The description to search for. */
	private String description;
	
	
	/**
	 * This Constructor will create a <code>ClientRequest</code>
	 * with clientid, name, type, location, owner and description set.
	 * 
	 * @param type the service type
	 * @param owner the owner of the service
	 * @param description the textual service description
	 * @param location the location of the service
	 * @param name the service name
	 */
	public ClientRequest(int clientId, String name, String type, String location, 
			String owner, String description) {
		super();
		this.clientID = clientId;
		this.name = name;
		this.type = type;
		this.location = location;
		this.owner = owner;
		this.description = description;
	}
	
	
	/**
	 * Gets a xml-string representation of the request
	 * <p>
	 * The returned string is a well formatted XML-string with the 
	 * following structure:
	 * <pre>
	 * &lt;ClientRequest ClientID=&quot;int&quot; 
	 *   Name=&quot;String&quot; 
	 *   Type=&quot;String&quot; 
	 *   Location=&quot;String&quot;
	 *   Owner=&quot;String&quot;
	 *   Description=&quot;String&quot;/&gt;
	 * </pre>
	 * @return the <code>String</code> with the request.
	 */
	public String getXML() {
		return "<ClientRequest ClientID=\"" + clientID +
				"\" Name=\"" + xmlEscape(name) +
				"\" Type=\"" + xmlEscape(type) +
				"\" Location=\"" + xmlEscape(location) +
				"\" Owner=\"" + xmlEscape(owner) +
				"\" Description=\"" + xmlEscape(description) + "\" />";
	}
	
	
	
	/**
	 * Gets the textual service description.
	 * 
	 * @return the textual service description
	 */
	public String getDescription() {
		return description;
	}
	
	
	
	/**
	 * Sets the textual service description.
	 * 
	 * @param description the textual service description
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	
	
	
	/**
	 * Gets the service location.
	 * 
	 * @return the service location
	 */
	public String getLocation() {
		return location;
	}
	
	
	
	/**
	 * Sets the service location.
	 * 
	 * @param location the service location
	 */
	public void setLocation(String location) {
		this.location = location;
	}
	
	
	
	/**
	 * Gets the service name.
	 * 
	 * @return the service name
	 */
	public String getName() {
		return name;
	}
	
	
	
	/**
	 * Sets the service name.
	 * 
	 * @param name the service name
	 */
	public void setName(String name) {
		this.name = name;
	}


	/**
	 * Gets the owner of the service.
	 * 
	 * @return the owner of the service
	 */
	public String getOwner() {
		return owner;
	}


	/**
	 * Sets the owner of the service.
	 * 
	 * @param owner the owner of the service
	 */
	public void setOwner(String owner) {
		this.owner = owner;
	}


	/**
	 * Gets the type of service.
	 * 
	 * @return the type of service
	 */
	public String getType() {
		return type;
	}


	/**
	 * Sets the type of service.
	 * 
	 * @param type the type of service
	 */
	public void setType(String type) {
		this.type = type;
	}
	
	
	/**
     * Replace special characters with XML escapes. 
     * XML has the following five predeclared entities:
     * <pre>
     * &amp; <small>(ampersand)</small> is replaced by &amp;amp;
     * &lt; <small>(less than)</small> is replaced by &amp;lt;
     * &gt; <small>(greater than)</small> is replaced by &amp;gt;
     * &apos; <small>(apostrophe)</small> is replaced by &amp;apos;
     * &quot; <small>(double quote)</small> is replaced by &amp;quot;
     * </pre>
     * @param string The string to be escaped.
     * @return The escaped string.
     */
    private String xmlEscape(String string) {
        if (string == null)
        	string = "";
    	StringBuffer sb = new StringBuffer();
        for (int i = 0, len = string.length(); i < len; i++) {
            char c = string.charAt(i);
            switch (c) {
            case '&':
                sb.append("&amp;");
                break;
            case '<':
                sb.append("&lt;");
                break;
            case '>':
                sb.append("&gt;");
                break;
            case '\'':
                sb.append("&apos;");
                break;
            case '"':
                sb.append("&quot;");
                break;
            default:
                sb.append(c);
            }
        }
        return sb.toString();
    }


	public int getClientID() {
		return clientID;
	}


	public void setClientID(int clientID) {
		this.clientID = clientID;
	}
	
}
