/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import no.ubicollab.osgi.service.discoverymanager.ServiceDiscovery;

/**
 * This class provides Service Discovery (SD) access for UbiCollab applications and tools. 
 * In addition this module is general enough to also be used by other applications 
 * which is able to connect through the soap interface.
 * <p>
 * The principal functionality for the SDM is to be the broker between 
 * Service Discovery (SD) client applications (consumers) and SD plugins
 * (producers). Client applications register their interests with the SDM and this
 * information is passed on to the SD plugins which will search for matching services.
 * The SD plugins will then register matching services in the queue that is assigned to
 * the SD client, and he will retrive them from here.
 * <p>
 * Both clients and plugins must register with the SDM, before they can enter or
 * retrieve any requests/interests. To maintain the principle of loose coupling, there
 * is no unregister methods. Instead, sessions will be terminated by timeout.
 * <p>
 * @author Kim-Steve Johansen
 * @version 0.6
 * @since 0.1
 */
public class DiscoveryManagerImpl implements ServiceDiscovery {
	
	
	/** The list that contain all registered plugins. */
	private PluginList pluginList;
	
	/** The list that contain all registered clients. */
	private ClientList clientList;
	

	/**
	 * The Constructor for this class. Instanciates the <code>ClientList</code>
	 * and <code>PluginList</code>, and starts the housekeeping thread.
	 */
	public DiscoveryManagerImpl() {
		pluginList = new PluginList();
		clientList = new ClientList();
		
		try {
			HouseKeeperThread housekeeper = new HouseKeeperThread(pluginList, clientList);
			housekeeper.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
	
	/**
	 * Gets a xml-string containing the client request list.
	 * <p>
	 * The returned string is a well formed XML-document with a structure
	 * as defined in <code>ServiceDiscovery#getClientRequest</code>
	 * 
	 * @see no.ubicollab.osgi.service.discoverymanager.ServiceDiscovery#getClientRequest
	 * 
	 * @param pluginID the plugin ID
	 * 
	 * @return the <code>String</code> with the client request list.
	 */
	public String getClientRequest(int pluginID) {
		return pluginList.getClientRequest(pluginID);
	}


	/**
	 * Method to return a list of the SD plugins which are registered with the SDM.
	 * Usefull for client who wishes to know which SD protocols they have support for.
	 * 
	 * @return a list of the SD plugins, formattet in XML. The following format is used:
	 * &lt;Pluginlist&gt;
	 * &lt;Plugin Name='' Id='' /&gt;
	 * &lt;/Pluginlist&gt;
	 */
	public String getSDPlugins() {
		return pluginList.getPluginList();
	}
	
	
	
	/**
	 * Return the result queue identified by the provided search request ID. After the 
	 * results are returned they will be removed from the queue.
	 * 
	 * @param clientID the client ID which point to the correct queue
	 * 
	 * @return The method returns "No matching services" if none are found (queue empty).
	 * "No such ID" if unknown or timed out ID. Otherwise a XML-formatted list of 
	 * services is returned.
	 */
	public String getServices(int clientID) {
		return clientList.getServiceQueue(clientID);
	}


	/**
	 * This method allows any authenticated user (consumer/client app) to register as 
	 * a client with the Service Discovery Manager (SDM). When a client is registered a 
	 * service queue is created and assigned to him. In this queue all services matching 
	 * requests from the client will be placed, as well as services discovered by user
	 * initiated physical SD plugins like RFID. 
	 * <p>
	 * Upon registration the client will also be assigned a unique ID which will be used 
	 * for identification when sending messages/communicating with the SDM. The client 
	 * will also be assigned a time-to-live which initially is set to 1 minute. Every 
	 * time the client invokes <code>getServices()</code> this time will be reset. If a
	 * client fails to connect to SDM for 1 minute, his session will be garbage 
	 * collected (by the housekeeper thread).
	 * 
	 * @return the unique ID to be used for identification when communicating with 
	 * the SDM.
	 */
	public int registerClient() {
		return clientList.createNewClient();
	}


	
	/**
	 * This method allows any authenticated user (client) to register a search request
	 * with the Service Discovery Manager. The manager will then make this request
	 * available to all registered service discovery plugins
	 * <p>
	 * After the request is placed, the client will get an ID which points to the queue
	 * of services found. This ID can then be used to poll for, and retrieve matching
	 * services using <code>getServices</code>.
	 * <p>
	 * When a request is placed the queue spawned will have a lifetime of 1 minute,
	 * and when the queue is queried a fresh minute is awarded.
	 * 
	 * @param owner the Owner of the service
	 * @param type the service type to search for
	 * @param description the description of the service
	 * @param location the location of the service
	 * @param clientID the unique client ID
	 * @param name the name of the service to search for
	 * 
	 * @return <code>true</code>, if client request is registered successfully,
	 *         <code>false</code> otherwise.
	 */
	public boolean registerClientRequest(int clientID, String name, String type, 
			String location, String owner, String description) {
		return pluginList.addClientRequest(new ClientRequest(clientID, name, type, location, owner, description));
	}
	
	
	
	/**
	 * Register a Service Discovery plugin with the Service Discovery Manager. When
	 * a plugin is registered it will get a unique ID which will be used for further
	 * communication. 
	 * <p>
	 * The registration will have a lifespan of 30 seconds, and will be
	 * reset to 30 second every time <code>getClientRequest()</code> or <code>
	 * registerService()</code> is invoked. When this time runs out the plugin 
	 * session will be garbage collected (by the housekeeper thread). 
	 * 
	 * @param protocolName the name of the protocol, e.g. UPnP or RFID
	 * @return the ID for identifying the plugin
	 * @see no.ubicollab.osgi.service.discoverymanager.ServiceDiscovery#registerSDPlugin(String)
	 */
	public int registerSDPlugin(String protocolName) {
		return pluginList.addNewPlugin(protocolName);
	}
	
	
	
	/**
	 * This method is provided for service discovery plugins to register services
	 * which matches client requests, with the Service Discovery Manager. The service
	 * will be added to the queue identified by the provided searchRequestID.
	 * <p>
	 * This method provides many options as parameters and all of these will not be
	 * applicable for each plugin. The ones not used should simply contain an
	 * empty string.
	 * 
	 * @param friendlyName The user-friendly name of the service
	 * @param descriptionURI URI for service description (e.g. the URL to a WSDL-file).
	 * @param type The service type (classification)
	 * @param owner The person/company/responsible for the service.
	 * @param description Tags (words or short sentences) describing the service.
	 * @param location Location where the service can be found
	 * @param searchRequestorID the ID for the search request queue to add this service to
	 * @param protocolID  The id for the service discovery protocol plugin.
	 * @param serviceURI The URI to invoke the service
	 * 
	 * @return True if the service is successfully registered, false if the client id
	 * 		   or protocol id does not exist.
	 * @see no.ubicollab.osgi.service.discoverymanager.ServiceDiscovery#registerService(int, int, String, String, String, String, String, String, String)
	 */
	public boolean registerService(int searchRequestorID, int protocolID, String descriptionURI, String serviceURI, String friendlyName, String type, String description, String owner, String location) {
		String sdProtocol = pluginList.getPluginName(protocolID);
		//Return false if invalid protocol id
		if ( sdProtocol.equals("Unknown ID") )
			return false;
		Service s = new Service(friendlyName, serviceURI, location, sdProtocol, owner, descriptionURI, type, description);
		return clientList.addServiceToQueue(searchRequestorID, s);
	}
	
	
	
	/**
	 * Read from the log and return selected log items in a XML-document, 
	 * stored in a <code>String</code>.
	 * 
	 * @param logLevel The log level to include in the report.
	 * @param numItems The number of log items to include in the report
	 * @return the <code>String</code> containg the resulting XML
	 * @see no.ubicollab.osgi.service.discoverymanager.ServiceDiscovery#getLogItems(int, int)
	 */
	public String getLogItems(int logLevel, int numItems) {
		Log log = Log.getInstance();
		return log.readFromLog(logLevel, numItems);
	}




}