/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.ArrayList;


/**
 * The Class <code>Plugin</code> is used to represent a service discovery plugin which 
 * have registered with the Service Discovery Manager. 
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class Plugin {
	
	/** The name of the plugin, e.g. UPnP or RFID */
	private String name;
	
	/** The unique id used to refrence the plugin. */
	private int id;
	
	/** The remaining time to live. */
	private int timeToLive;
	
	/** The list of client requests. */
	private ArrayList<ClientRequest> clientRequests;
	
	
	
	/**
	 * This Constructor is used to create a new plugin and give it a name and id.
	 * 
	 * @param name the plugin name
	 * @param id the unique id
	 */
	public Plugin(String name, int id) {
		super();
		this.name = name;
		this.id = id;
		timeToLive = GlobalSettings.PLUGIN_TIME_TO_LIVE;
		clientRequests = new ArrayList<ClientRequest>();
	}
	
	
	
	/**
	 * This constructor creates a new plugin object with no values set.
	 */
	public Plugin() {
		super();
		clientRequests = new ArrayList<ClientRequest>();
	}
	
	
	
	/**
	 * Adds a client request to the list of <code>ClientRequest</code> belonging
	 * to this <code>Plugin</code>.
	 * 
	 * @param clientRequest the client request to add
	 * 
	 * @return <code>true</code>, if the <code>ClientRequest</code> is
	 *         successfullt appended, <code>false</code> otherwise.
	 */
	public boolean addClientRequest(ClientRequest clientRequest) {
		return clientRequests.add(clientRequest);
	}
	
	
	
	/**
	 * Gets a xml-string containing the client request list.
	 * <p>
	 * The returned string is a well formatted XML-string with the 
	 * following structure:
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;ClientRequestList&gt;
	 *         &lt;ClientRequest ClientID=&quot;int&quot;
	 *            Name=&quot;String&quot; 
	 *            Type=&quot;String&quot; 
	 *            Location=&quot;String&quot;
	 *            Owner=&quot;String&quot;
	 *            Description=&quot;String&quot;/&gt;
	 *          ... more ClientRequest items ...<br/>
	 *      &lt;/ClientRequestList&gt;
	 * </pre>
	 * @return the <code>String</code> with the client request list.
	 */
	public String getClientRequestList() {
		String crList = GlobalSettings.XML_HEADER;
		crList += "<ClientRequestList>";
		while ( clientRequests.size() > 0 )
			crList += clientRequests.remove(0).getXML();
		crList += "</ClientRequestList>";
		return crList;
	}
	
	
	
	/**
	 * Gets the unique identifier for the plugin.
	 * 
	 * @return the unique id
	 */
	public int getId() {
		return id;
	}

	

	/**
	 * Sets the unique identifier for the plugin.
	 * 
	 * @param id the unique id
	 */
	public void setId(int id) {
		this.id = id;
	}



	/**
	 * Gets the plugins name.
	 * 
	 * @return the name of the plugin
	 */
	public String getName() {
		return name;
	}



	/**
	 * Sets the name of the plugin.
	 * 
	 * @param name the name of the plugin
	 */
	public void setName(String name) {
		this.name = name;
	}



	/**
	 * Sets the time to live.
	 * 
	 * @param timeToLive the time to live
	 */
	public void setTimeToLive(int timeToLive) {
		this.timeToLive = timeToLive;
	}



	/**
	 * Gets the remaining time to live for the plugin.
	 * 
	 * @return the time to live
	 */
	public int getTimeToLive() {
		return timeToLive;
	}
	
	
	/**
	 * Reset the time to live to a full lifetime, as defined by 
	 * <code>GlobalSettings</code>.
	 */
	public void resetTimeToLive() {
		timeToLive = GlobalSettings.PLUGIN_TIME_TO_LIVE;
	}
	
	
	
	/**
	 * Gets the XML representation for this plugin.
	 * <p>
	 * The format for the xml-string is as follows
	 * <pre>
	 * &lt;Plugin Name=&quot;String&quot; Id=&quot;int&quot; /&gt;
	 * </pre>
	 * @return the XML representation of this <code>Plugin</code> in a 
	 *         <code>string</code>
	 */
	public String getXML() {
		return "<Plugin Name='" + name + "' Id='" + id + "' />";
	}
	
}
