/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.ArrayList;

/**
 * The Class <code>PluginList</code> is used to represent all service discovery plugins which
 * have registered with the Service Discovery Manager. 
 * <p>
 * The class provide the methods neccessary to handle <code>Plugin</code> related tasks.
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class PluginList {

	/** The list containing all plugins. */
	private ArrayList<Plugin> pluginList;
	
	/** The <code>pluginCounter</code> is used to assign unique id&apos;s to plugins. */
	private int pluginCounter;
	
	
	
	/**
	 * The Constructor. Instanciates the plugin list and plugin counter.
	 */
	public PluginList() {
		super();
		pluginCounter = 0;
		pluginList = new ArrayList<Plugin>();
	}
	
	
	
	/**
	 * Adds a client request to the list of <code>ClientRequest</code> belonging
	 * to this <code>Plugin</code>.
	 * 
	 * @param clientRequest the client request to add
	 * 
	 * @return <code>true</code>, if the <code>ClientRequest</code> is
	 *         successfullt appended, <code>false</code> otherwise.
	 */
	public boolean addClientRequest(ClientRequest clientRequest) {
		boolean returValue = true;
		for (int i=0;i<pluginList.size();i++)
			if ( !pluginList.get(i).addClientRequest(clientRequest) )
				returValue = false;
		return returValue;
	}
	
	
	
	/**
	 * Gets a xml-string containing the client request list.
	 * <p>
	 * The returned string is a well formed XML-document with a structure
	 * as defined in <code>ServiceDiscovery#getClientRequest</code>
	 * 
	 * @see no.ubicollab.osgi.service.discoverymanager.ServiceDiscovery#getClientRequest
	 * 
	 * @param pluginID the plugin ID
	 * 
	 * @return the <code>String</code> with the client request list.
	 */
	public String getClientRequest(int pluginID) {
		String returValue = "";
		for (int i=0;i<pluginList.size();i++)
			if ( pluginList.get(i).getId() == pluginID ) {
				returValue = pluginList.get(i).getClientRequestList();
				pluginList.get(i).resetTimeToLive();
			}
		return returValue;
	}
	
	
	
	/**
	 * Creates a new plugin and returns the unique identifier for this plugin.
	 * 
	 * @return the int that is the unique id for the plugin.
	 */
	public int addNewPlugin(String name) {
		pluginCounter++;
		pluginList.add(new Plugin(name, pluginCounter));
		return pluginCounter;
	}
	
	
	/**
	 * Returns a list containing all registered plugins in XML-format.
	 * <p>
	 * The list returned is a well-formed xml-document stored in a
	 * <code>String</code>, with the following structure:
	 * <pre>
	 * &lt;Pluginlist&gt;
	 *   &lt;Plugin Name=&quot;String&quot; 
	 *   Id=&quot;int&quot; /&gt;<br/>
	 *   ... more Plugin elements ...<br/>
	 * &lt;/Pluginlist&gt;
	 * </pre>
	 * @return the XML-formatted <code>String</code> containing <code>Plugin</code> 
	 *         objects.
	 */
	public String getPluginList() {
		String xml = GlobalSettings.XML_HEADER;
		xml += "<Pluginlist>";
		for (int i=0;i<pluginList.size();i++)
			xml += pluginList.get(i).getXML();
		xml += "</Pluginlist>";
		return xml;
	}
	
	
	
	/**
	 * Method to get the name of a plugin based on the id.
	 * 
	 * @param pluginID the plugin ID to search for
	 * 
	 * @return the name of the plugin, or "Unknown ID" if none found
	 */
	public String getPluginName(int pluginID) {
		String pluginName = "Unknown ID";
		for (int i=0;i<pluginList.size();i++)
			if ( pluginList.get(i).getId() == pluginID )
				pluginName = pluginList.get(i).getName();
		return pluginName;
	}
	
	
	
	/**
	 * Decrease the time to live for all plugins. Plugins with time-to-live &lt;= 0 are
	 * assumed to have disconnected and will be removed.
	 * 
	 * @param time the time in millisecounds to decrease
	 */
	public synchronized void decreaseTimeToLive(int time) {
		int newTime;
		for (int i=0;i<pluginList.size();i++) {
			newTime = pluginList.get(i).getTimeToLive() - time;
			if ( newTime > 0 )
				pluginList.get(i).setTimeToLive( newTime );
			else {
				Plugin p = pluginList.remove(i);
				Log log = Log.getInstance();
				log.writeToLog("Plugin " + p.getName() + " timed out and was removed.", Log.DEBUG);
			}	
		}
	}
	
	
}
