/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

/**
 * The Service Class is intended to be a general class to hold information about a service
 * which have been found and registered by a plugin.
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class Service {
	
	/** The service discovery protocol who found the service */
	private String sdProtocol;
	
	/** The name of the service. */
	private String name;
	
	/** The service URI. */
	private String serviceURI;
	
	/** The description URI. */
	private String descriptionURI;
	
	/** The service type. */
	private String type;
	
	/** The location of the service. */
	private String location;
	
	/** The service owner.. */
	private String owner;
	
	/** The description for the service. */
	private String description;
	
	
	/**
	 * Constructor to create an empty Service object.
	 */
	public Service() {
		super();
	}
	
	
	/**
	 * Constructor for creating an Service object and setting the 
	 * describing parameters at the same time.
	 * 
	 * @param type the service type
	 * @param descriptionUri the service description uri
	 * @param owner the owner of the service
	 * @param description the textual service description
	 * @param sdProtocol the sd protocol that discovered the service
	 * @param location the location of the service
	 * @param name the name of the service
	 * @param serviceURI the service invocation URI
	 */
	public Service(String name, String serviceURI, String location, String sdProtocol, String owner, String descriptionUri, String type, String description) {
		super();
		this.sdProtocol = sdProtocol;
		this.name = name;
		this.serviceURI = serviceURI;
		this.location = location;
		this.owner = owner;
		this.descriptionURI = descriptionUri;
		this.type = type;
		this.description = description;
	}
	
	
	/**
	 * Gets the XML represenatation for the service.
	 * <p>
	 * The format of the returned xml-document contained in a
	 * <code>String</code> is as follows:
	 * <pre>
	 * &lt;Service Protocol=&quot;String&quot; 
	 *    Name=&quot;String&quot; 
	 *    ServiceUri=&quot;String&quot; 
	 *    Location=&quot;String&quot; 
	 *    Owner=&quot;String&quot; 
	 *    DescriptionUri=&quot;String&quot;
	 *    Type=&quot;String&quot; 
	 *    Description=&quot;String&quot;/&gt;<br/> 
	 * </pre>
	 * 
	 * @return the XML representing the service
	 */
	public String getXML() {
		String xml = "<Service ";
		xml += "Protocol=\"" + sdProtocol + "\" ";
		xml += "Name=\"" + name + "\" ";
		xml += "ServiceUri=\"" + serviceURI + "\" ";
		xml += "Location=\"" + location + "\" ";
		xml += "Owner=\"" + owner + "\" ";
		xml += "DescriptionUri=\"" + descriptionURI + "\" ";
		xml += "Type=\"" + type + "\" ";
		xml += "Description=\"" + description + "\" />";
		return xml;
	}
	
	
	
	
	
	/**
	 * Gets the name.
	 * 
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets the name.
	 * 
	 * @param name the name
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gets the service URI.
	 * 
	 * @return the service URI
	 */
	public String getServiceURI() {
		return serviceURI;
	}
	
	/**
	 * Sets the service URI.
	 * 
	 * @param serviceURI the service URI
	 */
	public void setServiceURI(String serviceURI) {
		this.serviceURI = serviceURI;
	}


	/**
	 * Gets the description URI.
	 * 
	 * @return the description URI
	 */
	public String getDescriptionURI() {
		return descriptionURI;
	}


	/**
	 * Sets the description URI.
	 * 
	 * @param descriptionURI the description URI
	 */
	public void setDescriptionURI(String descriptionURI) {
		this.descriptionURI = descriptionURI;
	}


	/**
	 * Gets the service type attribute.
	 * 
	 * @return the service type attribute
	 */
	public String getType() {
		return type;
	}


	/**
	 * Sets the service type attribute.
	 * 
	 * @param type the service type attribute
	 */
	public void setType(String type) {
		this.type = type;
	}


	/**
	 * Gets the description attribute for the service.
	 * 
	 * @return the description attribute for the service
	 */
	public String getDescription() {
		return description;
	}


	/**
	 * Sets the description attribute for the service.
	 * 
	 * @param description the description attribute for the service
	 */
	public void setDescription(String description) {
		this.description = description;
	}


	/**
	 * Gets the Service Discovery protocol attribute for the service.
	 * 
	 * @return the Service Discovery protocol attribute for the service
	 */
	public String getSdProtocol() {
		return sdProtocol;
	}


	/**
	 * Sets the Service Discovery protocol attribute for the service.
	 * 
	 * @param sdProtocol the Service Discovery protocol attribute for the service
	 */
	public void setSdProtocol(String sdProtocol) {
		this.sdProtocol = sdProtocol;
	}


	/**
	 * Gets the location attribute for the service.
	 * 
	 * @return the location attribute for the service
	 */
	public String getLocation() {
		return location;
	}


	/**
	 * Sets the location attribute for the service.
	 * 
	 * @param location the location attribute for the service
	 */
	public void setLocation(String location) {
		this.location = location;
	}


	/**
	 * Gets the owner attribute.
	 * 
	 * @return the owner attribute
	 */
	public String getOwner() {
		return owner;
	}


	/**
	 * Sets the service owner attribute.
	 * 
	 * @param owner the owner of the service
	 */
	public void setOwner(String owner) {
		this.owner = owner;
	}

	
}
