package no.ubicollab.osgi.service.discoverymanager;

/**
 * This is the interface exported by the UbiCollab Service Discovery Manager (SDM). The 
 * methods provides Service Discovery (SD) access for UbiCollab applications and tools. 
 * In addition this module is general enough to also be used by other applications 
 * which is able to connect through the soap interface.
 * <p>
 * The principal functionality for the SDM is to be the broker between 
 * Service Discovery (SD) client applications (consumers) and SD plugins
 * (producers). Client applications register their interests with the SDM and this
 * information is passed on to the SD plugins which will search for matching services.
 * The SD plugins will then register matching services in the queue that is assigned to
 * the SD client, and he will retrive them from here.
 * <p>
 * Both clients and plugins must register with the SDM, before they can enter or
 * retrieve any requests/interests. To maintain the principle of loose coupling, there
 * is no unregister methods. Instead, sessions will be terminated by timeout.
 * <p>
 * @author Kim-Steve Johansen
 * @version 0.6
 * @since 0.1
 */
public interface ServiceDiscovery {

	
	
	/**
	 * This method allows any authenticated user (consumer/client app) to register as 
	 * a client with the Service Discovery Manager (SDM). When a client is registered a 
	 * service queue is created and assigned to him. In this queue all services matching 
	 * requests from the client will be placed, as well as services discovered by user
	 * initiated physical SD plugins like RFID. 
	 * <p>
	 * Upon registration the client will also be assigned a unique ID which will be used 
	 * for identification when sending messages/communicating with the SDM. The client 
	 * will also be assigned a time-to-live which initially is set to 1 minute. Every 
	 * time the client invokes <code>getServices()</code> this time will be reset. If a
	 * client fails to connect to SDM for 1 minute, his session will be garbage 
	 * collected (by the housekeeper thread).
	 * 
	 * @return the unique ID to be used for identification when communicating with 
	 * the SDM.
	 */
	public int registerClient();
	
	
	/**
	 * This method allows any authenticated user (client) to register a search request
	 * with the Service Discovery Manager. The manager will then make this request
	 * available to all registered service discovery plugins
	 * <p>
	 * After the request is placed, the client will get an ID which points to the queue
	 * of services found. This ID can then be used to poll for, and retrieve matching
	 * services using <code>getServices</code>.
	 * <p>
	 * When a request is placed the queue spawned will have a lifetime of 1 minute,
	 * and when the queue is queried a fresh minute is awarded.
	 * 
	 * @param owner the Owner of the service
	 * @param type the service type to search for
	 * @param description the description of the service
	 * @param location the location of the service
	 * @param clientID the unique client ID
	 * @param name the name of the service to search for
	 * 
	 * @return <code>true</code>, if client request is registered successfully,
	 *         <code>false</code> otherwise.
	 */
	public boolean registerClientRequest(int clientID, String name, String type, 
			String location, String owner, String description);
	
	
	/**
	 * Return the result queue identified by the provided search request ID. After the 
	 * results are returned they will be removed from the queue.
	 * <p>
	 * The xml-structure used to represent the returned service list is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Servicelist&gt;
	 *         &lt;Service Protocol=&quot;String&quot; 
	 *            Name=&quot;String&quot; 
	 *            ServiceUri=&quot;String&quot; 
	 *            Location=&quot;String&quot; 
	 *            Owner=&quot;String&quot; 
	 *            DescriptionUri=&quot;String&quot;
	 *            Type=&quot;String&quot; 
	 *            Description=&quot;String&quot;/&gt;<br/>
	 *          ... more services ...<br/>
	 *      &lt;/Servicelist&gt;
	 * </pre>
	 * 
	 * @param clientID the client ID (used to select the correct queue)
	 * 
	 * @return The method returns "No matching services" if none are found (queue empty).
	 *         "No such ID" if unknown or timed out ID. Otherwise a XML-formatted list of 
	 *         services is returned.
	 */
	public String getServices(int clientID);

	
	/**
	 * Register a Service Discovery plugin with the Service Discovery Manager. When
	 * a plugin is registered it will get a unique ID which will be used for further
	 * communication. 
	 * <p>
	 * The registration will have a lifespan of 30 seconds, and will be
	 * reset to 30 second every time <code>getClientRequest()</code> or <code>
	 * registerService()</code> is invoked. When this time runs out the plugin 
	 * session will be garbage collected (by the housekeeper thread). 
	 * 
	 * @param protocolName the name of the protocol, e.g. UPnP or RFID
	 * 
	 * @return the ID for identifying the plugin
	 */
	public int registerSDPlugin(String protocolName);
	
	
	/**
	 * Method where SD plugins can get the interests/requests registered by 
	 * the SDM client(s).
	 * <p>
	 * The returned string is a well formatted XML-string with the 
	 * following structure:
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;ClientRequestList&gt;
	 *         &lt;ClientRequest ClientID=&quot;int&quot; 
	 *            Name=&quot;String&quot; 
	 *            Type=&quot;String&quot; 
	 *            Location=&quot;String&quot;
	 *            Owner=&quot;String&quot;
	 *            Description=&quot;String&quot;/&gt;
	 *          ... more ClientRequest items ...<br/>
	 *      &lt;/ClientRequestList&gt;
	 * </pre>
	 * 
	 * @param pluginID the plugin ID
	 * 
	 * @return A xml-formatted <code>String</code> containing the client request 
	 *         if any requests have been registered
	 */
	public String getClientRequest(int pluginID);
	
	
	/**
	 * This method is provided for service discovery plugins to register services
	 * which matches client requests, with the Service Discovery Manager. The service
	 * will be added to the queue identified by the provided searchRequestID.
	 * 
	 * This method provides many options as parameters and all of these will not be
	 * applicable for each plugin. The ones not used should simply contain an
	 * empty string.
	 * 
	 * @param friendlyName The user-friendly name of the service
	 * @param descriptionURI URI for service description (e.g. the URL to a WSDL-file).
	 * @param type The service type (classification)
	 * @param owner The person/company/responsible for the service.
	 * @param description Tags (words or short sentences) describing the service.
	 * @param location Location where the service can be found
	 * @param searchRequestorID the ID for the search request queue to add this service to
	 * @param protocolID  The id for the service discovery protocol plugin.
	 * @param serviceURI The URI to invoke the service
	 * 
	 * @return True if the service is successfully registered, false if the client id
	 * 		   or protocol id does not exist.
	 */
	public boolean registerService(int searchRequestorID, int protocolID, String descriptionURI,
			String serviceURI, String friendlyName, String type, String description, 
			String owner, String location);

	
	/**
	 * Method to return a list of the SD plugins which are registered with the SDM.
	 * Usefull for client who want to know which SD protocols they have support for.
	 * <p>
	 * The list returned is a well-formed xml-document, with the following structure:
	 * <pre>
	 * &lt;Pluginlist&gt;
	 *   &lt;Plugin Name=&quot;String&quot; 
	 *   Id=&quot;int&quot; /&gt;<br/>
	 *   ... more plugins ...<br/>
	 * &lt;/Pluginlist&gt;
	 * </pre>
	 * 
	 * @return a list contained in a <code>String</code> with the SD plugins.
	 */
	public String getSDPlugins();
	
	
	
	/**
	 * Read from the log and return selected log items in a XML-formatted string.
	 * The levels with equal or higher severity than what have been queried for
	 * will be retuned.
	 * <p>
	 * The log levels are defined as follows:<br/>
	 * <pre>
	 * Level - description
	 *   0   - DEBUG
	 *   1   - INFO
	 *   2   - WARNING
	 *   3   - ERROR
	 *   4   - FATAL
	 * <pre>
	 * <p>
	 * The xml-structure used to represent the returned log is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Log&gt;
	 *          &lt;Logitem Date=&quot;dd.mm.yyyy&quot; 
	 *            Time=&quot;hh:mm:ss&quot; 
	 *            LevelID=&quot;int&quot; 
	 *            Level=&quot;String&quot; 
	 *            Message=&quot;String&quot;/&gt;<br/> 
	 *          ... more log items ...<br/>
	 *      &lt;/Log&gt;
	 * </pre>
	 * 
	 * @param logLevel The log level to include in the report.
	 *      
	 * @param numItems The number of log items to include in the report
	 * 
	 * @return the <code>String</code> containg the resulting XML-document
	 */
	public String getLogItems(int logLevel, int numItems);
}
