/*
 * UbiCollab Service Domain Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is distributable under the Apache License v2.0 (http://www.apache.org/licenses/).
 * 
 * For more information, please contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import no.ubicollab.osgi.service.domainmanager.DomainManager;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;


/**
 * The Class <code>DomainManagerImpl</code> implements the <code>DomainManager</code>
 * interface and provides the neccessary functionality for service management on
 * UbiNodes. The public members of this class will be accessible through a SOAP/Web-
 * Service interface.
 * 
 * @author Kim-Steve Johansen
 * @version 0.6
 * @since 0.1
 */
public class DomainManagerImpl implements DomainManager {
	
	/** Reference to the <code>BundleContext</code> this bundle runs in. */
	private BundleContext bc;
	
	/** List of services */
	private ServiceList serviceList;
	
	
	
	/**
	 * This Constructor creates a new instance of <code>DomainManagerImpl</code>, and
	 * reads the service list from file.
	 * 
	 * @param bc the <code>BundleContext</code> in which this bundle runs.
	 */
	public DomainManagerImpl(BundleContext bc) {
		this.bc = bc;
		serviceList = new ServiceList(bc);
	}
	
	
	
	/**
	 * Removes the service provided by the bunde matching the provided bundleId.
	 * 
	 * @param bundleID the bundle ID of the service to remove
	 * 
	 * @return true, if service is removed, false othervise
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#removeService(java.lang.Long)
	 */
	public boolean removeService(Long bundleID) {
		Log log = Log.getInstance();
		boolean result = false;
		try {
			Bundle b = bc.getBundle(bundleID);
			b.uninstall();
			result = serviceList.removeService(bundleID);
		}
		catch (Exception e) {
			log.writeToLog("Failed to delete service identified by bundleId"
					+ bundleID, Log.ERROR);
			e.printStackTrace();
		}
		return result;
	}
	
	
	
	/**
	 * Searches for services installed on this UbiNode and returns handles to theese.
	 * <p>
	 * The xml-structure used to represent the returned service is described in
	 * <code>DomainManager#getServiceList</code>.
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#getServiceList
	 * @return The method returns a list of installed services
	 */
	public String getServiceList() {
		String sList = null;
		Log log = Log.getInstance();
		try {
			sList = serviceList.toXML();
			log.writeToLog("Servicelist fetched by client.", Log.INFO);
		} catch (Exception e) {
			log.writeToLog("Failed to fetch service list from database", Log.WARNING);
			e.printStackTrace();
		}
		return sList;
	}
	
	
	
	/**
	 * Installs a service. Use this method if you have the URL to a service bundle
	 * 
	 * @param friendlyName the friendly name of the service
	 * @param url The Url to the service bundle.
	 * 
	 * @return True if the service is successfully installed, false otherwise.
	 */
	public String installService(String url, String friendlyName) {
		Log log = Log.getInstance();
		try {
			Bundle b = bc.installBundle(url);
			String serviceUrl = "No Web-service";
			String descriptionUrl = "No Web-service";
			String serviceType = "no.ubicollab.osgi.core-service";
			String description = "No description provided";
			
			//Give the bundle a name
			if ( friendlyName == null )
				friendlyName = b.getHeaders().get("Bundle-Name").toString();
			else if ( friendlyName.equals("") )
				friendlyName = b.getHeaders().get("Bundle-Name").toString();
			
			//Get the desription
			if (b.getHeaders().get("Bundle-Description") != null)
				description = b.getHeaders().get("Bundle-Description").toString();
			
			//Extract information about proxy service
			if ( b.getRegisteredServices().length > 0 ) {
				if ( b.getRegisteredServices()[0].getProperty("SOAP.service.name") != null )
					serviceUrl = "/axis/services/" + b.getRegisteredServices()[0].getProperty("SOAP.service.name").toString();
				if ( b.getRegisteredServices()[0].getProperty("UbiCollab.service-proxy.name") != null )
					serviceType = "no.ubicollab.osgi.proxy-service";
			}
			
			//Fix descriptionUrl if service is an axis-enabled Web-service
			if ( !serviceUrl.equals(descriptionUrl) )
				descriptionUrl = serviceUrl + "?WSDL";
			
			serviceList.addService(new Service(friendlyName, serviceUrl, "null", 0, b.getBundleId(), serviceUrl+"?WSDL", serviceType, description) );
			
			log.writeToLog("Service: " + url + " successfully installed. ID=" + b.getBundleId(), Log.INFO);
			return "Success: Service installed. Check log for details.";
		} catch (BundleException e) {
			log.writeToLog("Failed to install service: " + url + ". Cause:" + e.getCause(), Log.WARNING);
			e.printStackTrace();
			return "Failure: Service could not be installed. Check log for details";
		}
		catch (Exception ex) {
			log.writeToLog("Failed to install service: " + url + ". Cause:" + ex.getCause(), Log.ERROR);
			ex.printStackTrace();
			return "Failure: Service could not be installed. Check log for details";
		}
	}
	
	
	
	/**
	 * Method to update the properties of the service identified by serviceUri.
	 * 
	 * @param spaceID the space ID for the space the service is placed in
	 * @param friendlyName The service friendly name
	 * @param type the type of service
	 * @param description the service description
	 * @param space The space this service will be active in
	 * @param bundleID The ID for the bundle which hosts the requested service
	 * 
	 * @return True if URI is found and update is OK, false otherwise.
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#updateServiceProperties(long, String, String, int, String, String)
	 */
	public boolean updateServiceProperties(long bundleID, String friendlyName, String space, int spaceID, String type, String description) {
		return serviceList.updateService(bundleID, friendlyName, space, spaceID, type, description);
	}
	
	
	
	/**
	 * Method to start a service installed on the UbiNode.
	 * 
	 * @param bundleID the bundle ID
	 * 
	 * @return True if the service is sucessfully started
	 */
	public boolean startService(Long bundleID) {
		boolean started = false;
		Bundle[] bundleList = bc.getBundles();
		Bundle b;
		for (int i=0;i<bundleList.length;i++) {
			b = bundleList[i];
			if ( b.getBundleId() == bundleID )
				try {
					b.start();
					started = true;
				} catch (BundleException e) {
					Log log = Log.getInstance();
					log.writeToLog("Failed to start service with ID=" + bundleID, Log.WARNING);
					e.printStackTrace();
				}
		}
		return started;
	}
	
	
	
	/**
	 * Method to stop a service installed on the UbiNode
	 * 
	 * @param bundleID the bundle ID
	 * @return True if the service is sucessfully started
	 */
	public boolean stopService(Long bundleID) {
		boolean stopped = false;
		Bundle[] bundleList = bc.getBundles();
		Bundle b;
		for (int i=0;i<bundleList.length;i++) {
			b = bundleList[i];
			if ( b.getBundleId() == bundleID )
				try {
					b.stop();
					stopped = true;
				} catch (BundleException e) {
					Log log = Log.getInstance();
					log.writeToLog("Failed to stop service with ID=" + bundleID, Log.WARNING);
					e.printStackTrace();
				}
		}
		return stopped;
	}
	
	
	
	/**
	 * Read from the log and return selected log items in a XML-formatted string.
	 * <p>
	 * The xml-structure used to represent the returned service is described in
	 * <code>DomainManager#getLogItems</code>.
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#getLogItems
	 * 
	 * @param logLevel The log level to include in the report.
	 * @param numItems The number of log items to include in the report
	 * 
	 * @return the string containg the resulting XML
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#getLogItems(int, int)
	 */
	public String getLogItems(int logLevel, int numItems) {
		Log log = Log.getInstance();
		return log.readFromLog(logLevel, numItems);
	}
	
	
	
	/**
	 * Gets all registered information about the specified service.
	 * <p>
	 * The xml-structure used to represent the returned service is described in
	 * <code>DomainManager#getInfoAboutService</code>.
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#getInfoAboutService
	 * @param bundleID the bundle ID the bundle is registered with.
	 * @return An <code>String</code> containg information about the service.
	 */
	public String getInfoAboutService(Long bundleID) {
		return serviceList.getInfoAboutService(bundleID);
	}
	
	
	
	/**
	 * Check if a given service exist.
	 * 
	 * @param uri the service uri
	 * 
	 * @return true, if service exists, false otherwise
	 */
	public boolean checkIfServiceExist(String uri) {
		return serviceList.checkIfServiceExist(uri);
	}
	
	
	
	/**
	 * Activates all services belonging to the space identified by the provided id.
	 * 
	 * @param SpaceID the ID for the Space to activate
	 * @param stopOtherServices true to stop the other service proxies, false othervise
	 * 
	 * @return true, if all proxies adhering to the space ID is started, false othervise.
	 */
	public boolean setActiveSpace(int SpaceID, boolean stopOtherServices) {
		return serviceList.setActiveSpace(SpaceID, stopOtherServices, this);
	}

}