/*
 * UbiCollab Service Domain Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.ArrayList;
import java.beans.XMLEncoder;
import java.beans.XMLDecoder;
import java.io.*;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import no.ubicollab.osgi.Service;


/**
 * The Class <code>ServiceList</code> is used to represent all services which
 * have been registered with the Service Domain Manager. 
 * <p>
 * The class functionality neccessary to handle <code>Service</code> management tasks.
 * 
 * @author Kim-Steve Johansen
 * @version 0.5
 * @since 0.1
 */
public class ServiceList {

	/** The service list. */
	ArrayList<Service> list = null;
	
	/** The refrence to the active <code>BundleContext</code>. */
	BundleContext bc;
	
	/** The file name for the service file. */
	String fileName;
	
	
	
	/**
	 * The Constructor for this class. Fetches the servicelist from file and
	 * initializes the service list.
	 * 
	 * @param bc the active Bundlecontext
	 */
	public ServiceList(BundleContext bc) {
		this.bc = bc;
		String drive = java.io.File.listRoots()[0].getAbsolutePath();
		fileName = drive + GlobalSettings.STORAGE_ROOT + System.getProperty("file.separator") + GlobalSettings.SERVICE_FILE;
		Log log = Log.getInstance();
		File path = new File(GlobalSettings.STORAGE_ROOT);
		boolean directoryOK = true;
		if ( !path.exists() ) {
			directoryOK = path.mkdirs();
		}
		if ( !directoryOK ) {
			log.writeToLog("ServiceList:Could not create root folder for UbiCollab. Folder=" + path.toString(), Log.ERROR);
			list = new ArrayList<Service>();
		}
		else {
			try {
				Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
				list = (ArrayList<Service>) readFromFile(fileName);
			} catch (Exception e) {
				log.writeToLog("ServiceList:Could not read from service list file. Filename=" + fileName, Log.WARNING);
				list = new ArrayList<Service>();
				e.printStackTrace();
			}
		}
	}
	
	
	/**
	 * Gets the state of the service proxy provided by the bundle identified by bundleID.
	 * 
	 * @param bundleID the bundle ID for the bundle providing the proxy service
	 * 
	 * @return the bundle state
	 */
	private String getBundleState(long bundleID) {
		Bundle b = bc.getBundle( bundleID );
		String bundleState = "unknown";
		switch (b.getState()) {
			case Bundle.ACTIVE: bundleState = "Started"; break;
			case Bundle.INSTALLED: bundleState = "Installed"; break;
			case Bundle.RESOLVED: bundleState = "Stopped"; break;
			case Bundle.STARTING: bundleState = "Starting"; break;
			case Bundle.STOPPING: bundleState = "Stopping"; break;
			case Bundle.UNINSTALLED: bundleState = "Uninstalled"; break;
		}
		return bundleState;
	}
	
	
	/**
	 * Method to update the properties of the service identified by serviceUri
	 * 
	 * @param bundleID The id for the bundle which hosts the service
	 * @param friendlyName The service friendly name
	 * @param space The space this service will be active in
	 * @return True if URI is found and update is OK, false otherwise.
	 */
	public boolean updateService(long bundleID, String friendlyName, String space, int spaceID, String type, String description) {
		boolean updateResult = false;
		Service s;
		Log log = Log.getInstance();
		for (int i=0;i<list.size();i++) {
			s = list.get(i);
			if (s.getBundleID() == bundleID) {
				s.setName(friendlyName);
				s.setSpace(space);
				s.setSpaceID(spaceID);
				s.setType(type);
				s.setDescription(description);
				persistenceUpdate();
				updateResult = true;
				log.writeToLog("Service information for: " + s.getName() + " was updated.", Log.INFO);
			}
		}
		if ( !updateResult )
			log.writeToLog("Attemt was made to update non-existant service. BundleID=" + bundleID, Log.WARNING);
		return updateResult;
	}
	
	
	
	
	/**
	 * Adds a specified service to the servicelist.
	 * 
	 * @param s the service to install
	 * 
	 * @return true, if service is added, false othervise
	 */
	public boolean addService(Service s) {
		boolean addResult = list.add(s);
		Log log = Log.getInstance();
		if (addResult) {
			persistenceUpdate();
			log.writeToLog("New service installed: " + s.getName() + " URL=" + s.getServiceURI(), Log.INFO);
		}
		else
			log.writeToLog("Failed to install service: " + s.getName() + " URL=" + s.getServiceURI(), Log.WARNING);
		return addResult;
	}
	
	
	/**
	 * Removes a service specified by the bundle id of the bundle which hosts the
	 * proxy service.
	 * 
	 * @param bundleID the bundle ID that is to be removed
	 * 
	 * @return true, if service is removed, false othervise
	 */
	public boolean removeService(Long bundleID) {
		boolean removeResult = false;
		Service s;
		Log log = Log.getInstance();
		for (int i=0;i<list.size();i++) {
			s = list.get(i);
			if (s.getBundleID() == bundleID) {
				if ( list.remove(s) ) {
					removeResult = true;
					persistenceUpdate();
					log.writeToLog("Service removed. Name=" + s.getName() + ". BundleID=" + bundleID, Log.INFO);
				}
			}
		}
		if ( !removeResult )
			log.writeToLog("Attemt was made to remove non-existant service. BundleID=" + bundleID, Log.WARNING);
		return removeResult;
	}
	
	
	
	/**
	 * Persistence update for the service xml file.
	 */
	public void persistenceUpdate() {
		Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
		Log log = Log.getInstance();
		try {			
			writeToFile(list, fileName);
		} catch (Exception e) {
			log.writeToLog("ServiceList: Unable to write XML file from servicelist. Filename=" + fileName, Log.WARNING);
			e.printStackTrace();
		}
	}
	
	
	private static void writeToFile(ArrayList sl, String filename) throws Exception {
        XMLEncoder encoder =
           new XMLEncoder(
              new BufferedOutputStream(
                new FileOutputStream(filename)));
        encoder.writeObject(sl);
        encoder.close();
    }

	
	private static ArrayList readFromFile(String filename) throws Exception {
        File f = new File(filename);
        ArrayList readObject;
        if ( f.exists() ) {
			XMLDecoder decoder = new XMLDecoder(new BufferedInputStream( new FileInputStream(filename)));
			readObject = (ArrayList) decoder.readObject();
			decoder.close();
		}
        else {
        	Log log = Log.getInstance();
        	log.writeToLog("Unabe to read from service xml file. Filename=" + filename, Log.ERROR);
        	readObject = new ArrayList<Service>();
        }
        return readObject;
    }

	
	
	/**
	 * Method to serialize the service list to XML. 
	 * <p>
	 * The xml-structure used to represent the returned service is described in
	 * <code>DomainManager#getServiceList</code>.
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#getServiceList
	 * 
	 * @return A <code>String</code> containing a representation of the service list
	 */
	public String toXML() {
		Service s;
		String xml = GlobalSettings.XML_HEADER;
		xml += "<Servicelist>";
		for (int i=0;i<list.size();i++) {
			s = list.get(i);
			xml += s.toXML( getBundleState( s.getBundleID() ) ); 
		}
		xml += "</Servicelist>";
		return xml;
	}
	
	
	/**
	 * Gets information about a service object, and returns it as serialized XML. 
	 * <p>
	 * The xml-structure used to represent the returned service is described in
	 * <code>DomainManager#getInfoAboutService</code>.
	 * 
	 * @see no.ubicollab.osgi.service.domainmanager.DomainManager#getInfoAboutService
	 * 
	 * @return A <code>String</code> containing a representation of the service list
	 *         or "INVALID BUNDLE ID" if bundle id does not exist.
	 */
	public String getInfoAboutService(Long bundleID) {
		Service s;
		boolean serviceFound = false;
		String xml = GlobalSettings.XML_HEADER;
		xml += "<Service>";
		for (int i=0;i<list.size();i++) {
			s = list.get(i);
			if ( s.getBundleID() == bundleID ) {
				serviceFound = true;
				xml += s.toXML( getBundleState( s.getBundleID() ) ); 
			}
		}
		xml += "</Service>";
		if ( serviceFound )
			return xml;
		else
			return "INVALID BUNDLE ID";
	}
	
	
	
	/**
	 * Check if a given service exist (is member of the service list).
	 * 
	 * @param uri the service uri
	 * 
	 * @return true, if service exists, false otherwise
	 */
	public boolean checkIfServiceExist(String uri) {
		for (int i=0;i<list.size();i++)
			if ( list.get(i).getServiceURI().equals(uri) ) 
				return true;
		return false;
	}
	
	
	
	/**
	 * Activates all services belonging to the space identified by the provided id.
	 * 
	 * @param SpaceID the ID for the Space to activate
	 * @param stopOtherServices true to stop the other service proxies, false othervise
	 * 
	 * @return true, if all proxies adhering to the space ID is started, false othervise.
	 */
	public boolean setActiveSpace(int SpaceID, boolean stopOtherServices, DomainManagerImpl dm) {
		boolean result = true;
		for (int i=0;i<list.size();i++) {
			if ( list.get(i).getSpaceID() == SpaceID ) 
				if ( !dm.startService( list.get(i).getBundleID() ) )
					result = false;
			else
				if ( stopOtherServices )
					if ( !dm.stopService( list.get(i).getBundleID() ) )
						result = false;
		}
		return result;
	}
	
	
}
