/*
 * UbiCollab Service Domain Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is distributable under the Apache License v2.0 (http://www.apache.org/licenses/).
 * 
 * For more information, please contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi.service.domainmanager;


/**
 * An interface for UbiCollab Service Domain Manager. Provides management functionality 
 * for services/bundles through web service calls.
 * 
 * @author Kim Steve Johansen
 * @version 0.9
 */
public interface DomainManager {

	
	/**
	 * Return a list of the services which have been installed on this UbiNode through 
	 * the Service Domain Manager. 
	 * <p>
	 * The xml-structure used to represent the returned service list is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Servicelist&gt;
	 *          &lt;Service ID=&quot;int&quot; 
	 *            ServiceUri=&quot;String&quot; 
	 *            Name=&quot;String&quot; 
	 *            Space=&quot;String&quot;
	 *            SpaceID=&quot;int&quot;
	 *            State=&quot;String&quot; 
	 *            DescriptionUri=&quot;String&quot; 
	 *            Type=&quot;String&quot; 
	 *            Description=&quot;String&quot;/&gt;<br/> 
	 *            ... more services ...<br/>
	 *      &lt;/Servicelist&gt;
	 * </pre>
	 * 
	 * @return The method returns a <code>String</code> with a XML-document 
	 *         which contains all installed services.    
	 */
	public String getServiceList();
	
	
	
	/**
	 * Gets all registered information about the specified service.
	 * <p>
	 * The xml-structure used to represent the returned service is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Service&gt;
	 *          &lt;Service ID=&quot;int&quot; 
	 *            ServiceUri=&quot;String&quot; 
	 *            Name=&quot;String&quot; 
	 *            Space=&quot;String&quot;
	 *            SpaceID=&quot;int&quot;
	 *            State=&quot;String&quot; 
	 *            DescriptionUri=&quot;String&quot; 
	 *            Type=&quot;String&quot; 
	 *            Description=&quot;String&quot;/&gt;<br/> 
	 *      &lt;/Service&gt;
	 * </pre>
	 * @param bundleID the bundle ID the bundle is registered with.
	 * 
	 * @return An xml-encoded <code>String</code> containg information about the service,
	 *         or "INVALID BUNDLE ID" if invalid bundle id is provided.
	 */
	public String getInfoAboutService(Long bundleID);
	
	
	
	/**
	 * Installs a service. Use this method if you have the URL to a service bundle
	 * 
	 * @param friendlyName the friendly name of the service
	 * @param url The Url to the service bundle.
	 * 
	 * @return A text describing the outcome of the installation.
	 */
	public String installService(String url, String friendlyName);
	
	
	
	/**
	 * Check if a given service exist.
	 * 
	 * @param uri the service uri
	 * 
	 * @return true, if service exists, false otherwise
	 */
	public boolean checkIfServiceExist(String uri);
	
	
	
	/**
	 * Removes the service proxy which is provided through the bundle represented by bundleID.
	 * 
	 * @param bundleID The URI for the service which is to be removed
	 * @return <code>true<code> if the service exist and are successfully removed, 
	 *         <code>false<code> othervise
	 */
	public boolean removeService(Long bundleID);
	
	
	
	/**
	 * Method to update the properties of the service identified by serviceUri.
	 * 
	 * @param friendlyName The service friendly name
	 * @param description the textual description of the service
	 * @param space The space this service will be active in
	 * @param bundleID the bundle ID
	 * 
	 * @return <code>true<code> if URI is found and update is OK, 
	 *         <code>false<code> othervise
	 */
	public boolean updateServiceProperties(long bundleID, String friendlyName, String space, int spaceID, String type, String description);

	
	
	/**
	 * Method to start a service installed on the UbiNode.
	 * 
	 * @param bundleID the bundle ID for the bundle providing the service
	 * 
	 * @return <code>true<code> if the service is sucessfully started, 
	 *         <code>false<code> othervise
	 */
	public boolean startService(Long bundleID);
	
	
	
	/**
	 * Method to stop a service installed on the UbiNode
	 * 
	 * @param bundleID the bundle ID for the bundle providing the service
	 * @return <code>true<code> if the service is sucessfully stopped, 
	 *         <code>false<code> othervise
	 */
	public boolean stopService(Long bundleID);
	
	
	
	/**
	 * Read from the log and return selected log items in a XML-formatted string.
	 * <p>
	 * The xml-structure used to represent the returned log is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Log&gt;
	 *          &lt;Logitem Date=&quot;dd.mm.yyyy&quot; 
	 *            Time=&quot;hh:mm:ss&quot; 
	 *            Level=&quot;String&quot; 
	 *            Message=&quot;String&quot;/&gt;<br/> 
	 *          ... more log items ...<br/>
	 *      &lt;/Log&gt;
	 * </pre>
	 * 
	 * @param logLevel The log level to include in the report. Valid arguments are:
	 *      'DEBUG', 'INFO', 'WARNING', 'ERROR', 'FATAL' and 'ALL' for all types.
	 * @param numItems The number of log items to include in the report
	 * 
	 * @return the <code>String</code> containg the resulting XML
	 */
	public String getLogItems(int logLevel, int numItems);
	
	
	/**
	 * Activates all services belonging to the space identified by the provided id. 
	 * Also provides the option to stop the other services.
	 * 
	 * @param SpaceID the ID for the Space to activate
	 * @param stopOtherServices true to stop the other service proxies, false othervise
	 * 
	 * @return <code>true<code> if all proxies adhering to the space ID is started, 
	 *         <code>false<code> othervise
	 */
	public boolean setActiveSpace(int SpaceID, boolean stopOtherServices);
	
}
