using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using ServiceManagementTool.CSManager;
using System.Web.Services.Protocols;
using ServiceManagementTool.UbiCollab;

namespace ServiceManagementTool
{
    /// <summary>
    /// This class provides a thread that will register a client with the Collaboration 
    /// Space Manager, and check if any services matching requests are found.
    /// </summary>
    public class CollaborationSpaceThread
    {
        #region Members

        //An instance of the SAX XML-parser
        private XMLHandler xmlHandler;

        /* Service specific URL-part, for the Collaboration Space */
        private string csManagerURL = "/axis/services/CSManager?wsdl";

        // Main thread sets this event to stop worker thread:
        private ManualResetEvent s_EventStop;

        // Worker thread sets this event when it is stopped:
        private ManualResetEvent s_EventStopped;

        // Reference to main form, used to make syncronous user interface calls:
        private ServiceGUI s_form;

        /* Reference to the CI Manager */
        private CSManagerImplService csManager;

        /* Is CI Manager connected? */
        private bool csManagerConnected = false;

        /* List of existing Collaboration Spaces */
        private ArrayList collabSpaceList = new ArrayList();

        #endregion


        #region Functions


        /// <summary>
        /// This class connects to the collaboration space manager identified
        /// by the hostname provided.
        /// </summary>
        /// <param name="eventStop"></param>
        /// <param name="eventStopped"></param>
        /// <param name="form"></param>
        /// <param name="hostname">Assumes at hostname of type http://hostname:port</param>
        /// <param name="userName">UbiCollab user name</param>
        public CollaborationSpaceThread(ManualResetEvent eventStop,
                           ManualResetEvent eventStopped,
                           ServiceGUI form, string hostname, string userName)
        {
            s_EventStop = eventStop;
            s_EventStopped = eventStopped;
            s_form = form;
            xmlHandler = new XMLHandler();

            try
            {
                csManager = new CSManagerImplService();
                csManager.Url = hostname + csManagerURL;
                csManager.login(userName);
                csManagerConnected = true;
            }
            catch (Exception ex)
            {
                csManagerConnected = false;
                MessageBox.Show("Unable to connect to Collaboration Space Manager (" + hostname + ").\n Message: " + ex.Message);
                Console.Out.WriteLine(ex.StackTrace);
            }
        }



        /// <summary>
        /// The code within the Run method is the one which runs as a thread
        /// </summary>
        public void Run()
        {
            int counter = 14;

            while ( csManagerConnected )
            {
                counter++;

                if (counter == 15)
                {
                    counter = 0;

                    //get services if any
                    try
                    {
                        string collabSpaces = csManager.getCsString();
                        string[] spaceItems;

                        foreach (string collabSpaceItem in collabSpaces.Split(char.Parse("\n")))
                        {
                            spaceItems = collabSpaceItem.Split(char.Parse("_"));
                            if (spaceItems.Length > 2)
                            {
                                collabSpaceList.Add(new CollabSpace(spaceItems[0], spaceItems[1]));
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Console.Out.WriteLine(ex.StackTrace);
                        csManagerConnected = false;
                    }

                    // Make a synchronous call to main form to update collaboration 
                    // spaces found, if any.
                    if (collabSpaceList.Count > 0)
                        s_form.Invoke(s_form.s_DelegateUpdateSpaces, new Object[] { collabSpaceList });

                }

                // check if thread is cancelled
                if (s_EventStop.WaitOne(0, true))
                {
                    s_EventStopped.Set(); // inform main thread that this thread stopped
                    return;
                }

                Thread.Sleep(1000);

            }

            // Make asynchronous call to main form
            // to inform it that thread finished
            s_form.Invoke(s_form.s_DelegateCIThreadFinished, null);
        }

        #endregion











        #region Properties handlers



        #endregion

    }
}
