using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using ServiceManagementTool.discoveryManager;
using System.Windows.Forms;

namespace ServiceManagementTool
{
    /// <summary>
    /// This class provides a thread that will register a client with the Service 
    /// Domain Manager, and check if any services matching requests are found.
    /// </summary>
    public class ServiceDiscoveryThread
    {
        
        #region Class members

        /* An instance of the SAX XML-parser */
        private XMLHandler xmlHandler;

        /* URL to the Discovery Manager, with a default value */
        private string discoveryManagerURL = "/axis/services/discoveryManager";

        /* Main thread sets this event to stop worker thread */
        ManualResetEvent m_EventStop;

        /* Worker thread sets this event when it is stopped */
        ManualResetEvent m_EventStopped;

        /* Reference to main form, used to make syncronous user interface calls */
        private ServiceGUI m_form;

        /* Web Reference to the Service Discovery Manager */
        private DiscoveryManagerImplService discoveryManager;

        /* The client subscriber id at the Service Discovery Manager */
        int clientId;

        #endregion



        #region Class Functions

        /// <summary>
        /// This class will register a client request for services matching a set 
        /// of criteria, as specified by the parameters.
        /// </summary>
        /// <param name="txtFindName"></param>
        /// <param name="txtFindType"></param>
        /// <param name="txtFindLocation"></param>
        /// <param name="txtFindOwner"></param>
        /// <param name="txtFindDescription"></param>
        public void getServices(string txtFindName, string txtFindType, 
            string txtFindLocation, string txtFindOwner, string txtFindDescription)
        {
            discoveryManager.registerClientRequest(clientId, txtFindName, txtFindType,
                txtFindLocation, txtFindOwner, txtFindDescription);
        }


        /// <summary>
        /// This class will fetch log items, formatted in a xml-document from the
        /// Service Discovery Service.
        /// </summary>
        /// <param name="logLevel">The log level, from 0..4</param>
        /// <param name="logItemCount">How many items to return</param>
        /// <returns>string containing an xml-document</returns>
        public string getLogItems(int logLevel, int logItemCount)
        {
            try
            {
                return discoveryManager.getLogItems(logLevel, logItemCount);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Unable to retrieve log from Service Manager.\nMessage: " + ex.Message);
            }
            return "<?xml version=\"1.0\"?><Log></Log>";
        }



        /// <summary>
        /// Constructor which initializes the thread events, connection to Service
        /// Discovery Manager and other needed functionality, like xmlHandler.
        /// </summary>
        /// <param name="eventStop"></param>
        /// <param name="eventStopped"></param>
        /// <param name="form"></param>
        public ServiceDiscoveryThread(ManualResetEvent eventStop,
                           ManualResetEvent eventStopped,
                           ServiceGUI form, string serviceHost)
        {
            m_EventStop = eventStop;
            m_EventStopped = eventStopped;
            m_form = form;
            discoveryManager = new DiscoveryManagerImplService();
            if (serviceHost == null || serviceHost == "")
                discoveryManager.Url = "http://localhost:8080" + discoveryManagerURL;
            else
                discoveryManager.Url = serviceHost + discoveryManagerURL;
            xmlHandler = new XMLHandler();
        }

        /// <summary>
        /// The code within the Run method is the one which runs as a thread
        /// </summary>
        public void Run()
        {
            //pCounter will count up to 5, then get a list of supported sd protocols
            int pCounter = 0;
            //Stores the getService call result
            String s = "";
            //Stores the getSDProtocols result
            String p = "";

            //Register this client with the Service Discovery Manager and register ID
            //with main class to enable searches on this id.
            try
                {
                clientId = discoveryManager.registerClient();
                m_form.Invoke(m_form.m_DelegateSetServiceDiscoverySubscriptionID, new Object[] { clientId });
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(ex.StackTrace);
                clientId = -1;
            }

            while (clientId != -1)
            {
                pCounter++;

                //get services if any
                try
                {
                    s = discoveryManager.getServices(clientId);
                }
                catch (Exception ex)
                {
                    Console.Out.WriteLine(ex.StackTrace);
                    clientId = -1;
                }
                
                // check if thread is cancelled, befor GUI updates are performed
                if (m_EventStop.WaitOne(0, true))
                {
                    m_EventStopped.Set(); // inform main thread that this thread stopped
                    return;
                }

                //try to re-register with discovery manager if subscription is outdated
                if ( s.Equals("No such ID") )
                {
                    clientId = discoveryManager.registerClient();
                    m_form.Invoke(m_form.m_DelegateSetServiceDiscoverySubscriptionID, new Object[] { clientId });
                }

                // Make a synchronous call to main form to update services found, if any.
                if ( !s.Equals("No matching services"))
                    m_form.Invoke(m_form.m_DelegateAddServicesFound, new Object[] { s });

                //update supported protocols, run only every three tick
                if (pCounter == 3)
                {
                    pCounter = 0;
                    p = xmlHandler.parseSDProtocolList( discoveryManager.getSDPlugins() );
                    m_form.Invoke(m_form.m_DelegateSetServiceDiscoveryProtocols, new Object[] { p });
                }

                // check if thread is cancelled
                if (m_EventStop.WaitOne(0, true))
                {
                    m_EventStopped.Set(); // inform main thread that this thread stopped
                    return;
                }

                Thread.Sleep(1500);
            }

            // Make asynchronous call to main form
            // to inform it that thread finished
            m_form.Invoke(m_form.m_DelegateThreadFinished, null);
        }

        #endregion











        #region Properties handlers

        /// <summary>
        /// This propery is the URL to the Discovery Manager, and it's value will be used 
        /// when connecting to it.
        /// </summary>
        public string DiscoveryManagerURL
        {
            get { return discoveryManagerURL; }
            set { discoveryManagerURL = value; }
        }

        #endregion

    }
}
