using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Xml;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ServiceManagementTool.ubiHome;

namespace ServiceManagementTool.UbiCollab
{
    public partial class PublishAssistant : Form
    {
        /* The service running at UbiHome */
        public Service ubiHomeService;

        /* List of existing Collaboration Instances */
        private ArrayList collabInstanceList = new ArrayList();

        /* List of currently published services */
        private ArrayList publishedServiceList = new ArrayList();

        /* List of CIs where user is member */
        private ArrayList memberList = new ArrayList();

        /* List of CIs where user is not member */
        private ArrayList nonMemberList = new ArrayList();

        /* The username for the logged in user */
        private string userName;

        /* The Service identifier */
        private int serviceId;
        
        
        public PublishAssistant(string userName, int serviceId, string serviceName, string ubiHome)
        {
            InitializeComponent();
            ubiHomeService = new Service();
            ubiHomeService.Url = ubiHome;

            this.userName = userName;
            this.serviceId = serviceId;

            txtService.Text = serviceName;

            //Fetch initial data from Collaboration Instance
            collabInstanceList = getCollaborationInstances(userName);
            publishedServiceList = getPublishedServices(serviceId, userName);

            //populate nonMemberlist and memberList from matching 
            populateLists();

            updateListBoxes();
        }



        private void updateListBoxes()
        {
            lbNotMember.DataSource = null;
            lbMember.DataSource = null;
            if (nonMemberList.Count > 0)
            {
                lbNotMember.DataSource = nonMemberList;
                lbNotMember.DisplayMember = "CollabName";
                lbNotMember.ValueMember = "CollabId";
            }
            if (memberList.Count > 0)
            {
                lbMember.DataSource = memberList;
                lbMember.DisplayMember = "CollabName";
                lbMember.ValueMember = "CollabId";
            }
        }



        private void populateLists()
        {
            foreach (CollabInstance ci in collabInstanceList)
            {
                bool found = false;
                foreach (PublishedServices ps in publishedServiceList)
                    if (ps.CollabInstanceID == ci.CollabId)
                        found = true;
                if (found)
                    memberList.Add(ci);
                else
                    nonMemberList.Add(ci);

            }
        }


        private ArrayList getCollaborationInstances(string userName)
        {
            ArrayList collabInstanceList = new ArrayList();
            string ciListXML = ubiHomeService.getAll(userName);
            string ccid = "";
            string name = "";

            // Read xml-string
            XmlTextReader textReader = new XmlTextReader(new System.IO.StringReader(ciListXML));

            // Read through all elements in string.
            while (textReader.Read())
            {
                XmlNodeType nType = textReader.NodeType;

                if (textReader.NodeType == XmlNodeType.Element && textReader.Name == "CIID")
                    ccid = textReader.ReadElementContentAsString();

                if (nType == XmlNodeType.Element && textReader.Name == "name")
                    name = textReader.ReadElementContentAsString();

                if (ccid != "" && name != "")
                {
                    collabInstanceList.Add(new CollabInstance(ccid, name));
                    ccid = "";
                    name = "";
                }
            }
            return collabInstanceList;
        }

        
        /// <summary>
        /// This method fetches all services published by a user, and return an
        /// ArrayList of PublishedServices with the ones matching the provided serviceId
        /// </summary>
        /// <param name="serviceID"></param>
        /// <param name="userName"></param>
        private ArrayList getPublishedServices(int serviceID, string userName)
        {
            ArrayList publishedServiceList = new ArrayList();
            string collabSpaces = ubiHomeService.getPublishedServicesUser(userName);

            //split array of strings to single strings
            foreach (string collabSpaceItem in collabSpaces.Split(char.Parse("\n")))
            {
                string[] publishedItems = collabSpaceItem.Split(char.Parse(","));
                
                if (publishedItems.Length > 2)
                {
                    //Only add publications for this service
                    if (Int32.Parse(publishedItems[1]) == serviceID)
                    {
                        publishedServiceList.Add(new PublishedServices( publishedItems[1],
                            publishedItems[0], publishedItems[2]));
                    }
                }
            }
            return publishedServiceList;
        }



        /// <summary>
        /// Handler for the publish button. Will fetch the service selected for publishing,
        /// publish this and then update the GUI.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPublish_Click(object sender, EventArgs e)
        {
            int index = lbNotMember.SelectedIndex;
            if (index != -1)
            {
                CollabInstance ci = (CollabInstance) nonMemberList[index];
                ubiHomeService.publishService(serviceId+"", ci.CollabId, userName);
                nonMemberList.RemoveAt(index);
                memberList.Add(ci);
                updateListBoxes();
            }
            else
                MessageBox.Show("You must first select a Collaboration Instance to publish the service in.");
        }

        
        
        /// <summary>
        /// Handler for the unpublish button. Will fetch the service selected for unpublishing,
        /// unpublish this and update the GUI.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUnPublish_Click(object sender, EventArgs e)
        {
            int index = lbMember.SelectedIndex;
            if (index != -1)
            {
                CollabInstance ci = (CollabInstance)memberList[index];
                ubiHomeService.unpublishService(serviceId + "", ci.CollabId, userName);
                memberList.RemoveAt(index);
                nonMemberList.Add(ci);
                updateListBoxes();
            }
            else
                MessageBox.Show("You must first select a Collaboration Instance to unpublish the service in.");
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }


    }
}