/*
 * UbiCollab Service Registry is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is distributable under the Apache License v2.0 (http://www.apache.org/licenses/).
 * 
 * For more information, please contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * The Class DBConnection offers methods to open/close database connections, and
 * also the read/write methods that is required by Service Registry.
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class DBConnection {

	/** The handle to the sql connection. */
	private Connection sqlConn = null;

	/** Gets the corrects newline character for the system the bundle runs on. */
	public static String newline = System.getProperty("line.separator");
	
	
	
	/**
	 * This Constructor will create a new DBConnection instance. The actual db
	 * connection is opened at the moment the class is instanciated.
	 */
	public DBConnection() {
		Log log = Log.getInstance();
		try {
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			sqlConn = connectToDatabase();
			log.writeToLog("Connected to DB", Log.DEBUG);
		} 
		catch (Exception e) {
			log.writeToLog("Unabe to connect to database. Message:"
					+ e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Connect to the database and returns the connection handle.
	 * 
	 * @return the connection handle
	 */
	private Connection connectToDatabase() {
		try {
			sqlConn = DriverManager.getConnection(
					GlobalSettings.CONNECTION_STRING,
					GlobalSettings.CONNECTION_USERNAME,
					GlobalSettings.CONNECTION_PASSWORD);
		} catch (SQLException e) {
			Log log = Log.getInstance();
			log.writeToLog("Unabe to connect to database. Message:"
					+ e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		return sqlConn;
	}

	
	
	/**
	 * Disconnects from database. This method is ment to be called when the bundle
	 * is stopped.
	 */
	public void disconnectFromDatabase() {
		Log log = Log.getInstance();
		try {
			sqlConn.close();
			log.writeToLog("Disconnected from DB", Log.DEBUG);
		} catch (Exception e) {
			log.writeToLog("Unabe to disconnect from database. Message:"
					+ e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
	}
	
	
	
	/**
	 * Adds a new service record to the database.
	 * 
	 * @param type Classification of service (in service type hierarchy)
	 * @param descriptionURI The description URI for the service
	 * @param owner The owner or responible for the service
	 * @param description the textual service description
	 * @param location The location of the service
	 * @param name A selected name the user should be able to understand
	 * @param serviceURI The Url to the service bundle.
	 * 
	 * @return <code>true</code> if the service refrence is successfully added, 
	 *         <code>false</code> otherwise.
	 */
	public boolean addService(String serviceURI, String descriptionURI, String name, String type, String location, String owner, String description) {
		//Create some variables 
		Statement s = null;
		int count;
		
		//Create insert statement
		String sql = "INSERT INTO ServiceRegistry (ServiceURI, DescriptionURI, Name, Type, Location, Owner, Description)";
		sql += " VALUES ('" + getDBInsertFormat(serviceURI) + "', '";
		sql += getDBInsertFormat(descriptionURI) + "', '";
		sql += getDBInsertFormat(name) + "', '";
		sql += getDBInsertFormat(type) + "', '";
		sql += getDBInsertFormat(location) + "', '";
		sql += getDBInsertFormat(owner) + "', '";
		sql += getDBInsertFormat(description) + "')";
		
		//Run query
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			s = sqlConn.createStatement();
			count = s.executeUpdate(sql);
			sqlConn.close();
		} 
		catch (SQLException e) {
			count = 0;
			Log log = Log.getInstance();
			log.writeToLog("Unabe to connect to database. Message:"
					+ e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		
		if ( count == 0 )
			return false;
		else
			return true;
	}
	
	
	
	/**
	 * Updates a service record to the database, based on the <code>serviceID</code>.
	 * 
	 * @param serviceID the service ID of the service to update
	 * @param descriptionURI The description URI for the service
	 * @param type Classification of service (in service type hierarchy)
	 * @param owner The owner or responible for the service
	 * @param description the textual service description
	 * @param location The location of the service
	 * @param name A selected name the user should be able to understand
	 * @param serviceURI The Url to the service bundle.
	 * 
	 * @return <code>true</code> if the service refrence is successfully updated,
	 *         <code>false</code> otherwise.
	 */
	public boolean updateService(int serviceID, String serviceURI, String descriptionURI, String name, String type, String location, String owner, String description) {
		//Create some variables 
		Statement s = null;
		int count;
		
		//Create update statement
		String sql = "UPDATE ServiceRegistry SET ";
		sql += "ServiceURI = '" + getDBInsertFormat(serviceURI) + "', ";
		sql += "DescriptionURI = '" + getDBInsertFormat(descriptionURI) + "', ";
		sql += "Name = '" + getDBInsertFormat(name) + "', ";
		sql += "Type = '" + getDBInsertFormat(type) + "', ";
		sql += "Location = '" + getDBInsertFormat(location) + "', ";
		sql += "Owner = '" + getDBInsertFormat(owner) + "', ";
		sql += "Description = '" + getDBInsertFormat(description) + "' ";
		sql += "WHERE ID = " + serviceID;
				
		//Run query
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			s = sqlConn.createStatement();
			count = s.executeUpdate(sql);
			sqlConn.close();
		} 
		catch (SQLException e) {
			count = 0;
			Log log = Log.getInstance();
			log.writeToLog("Unable to update service with ID=" + serviceID 
					+ ". Message:" + e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		
		if ( count == 0 )
			return false;
		else
			return true;
	}
	
	
	
	/**
	 * Deletes a service refrence record from the database.
	 * 
	 * @param serviceID the service ID for the service to remove
	 * 
	 * @return <code>true</code> if the service refrence is successfully updated,
	 *         <code>false</code> otherwise.
	 */
	public boolean deleteService(int serviceID) {
		//Create some variables 
		Statement s = null;
		int count;
		
		//Create delete statement
		String sql = "DELETE FROM ServiceRegistry WHERE ID = " + serviceID;
		
		//Run query
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			s = sqlConn.createStatement();
			count = s.executeUpdate(sql);
			sqlConn.close();
		} 
		catch (SQLException e) {
			count = 0;
			Log log = Log.getInstance();
			log.writeToLog("Unabe to delete service from database. Message:"
					+ e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		
		if ( count == 0 )
			return false;
		else
			return true;
	}
	
	
	
	/**
	 * Get a list of services matching the specified query parameters.
	 * 
	 * @param type The type of service
	 * @param owner The owner of the srvice
	 * @param description the description of the service
	 * @param location The location of the service
	 * @param name The name of the service
	 * 
	 * @return An XML-string string containg the query results if any matches are
	 * found, "NO SERVICES MATCHES THE QUERY" otherwise.
	 */
	public String getServiceList(String name, String type, String location, String owner, String description) {
		//Create some variables 
		String result = GlobalSettings.XML_HEADER + newline + " <Servicelist>" + newline;
		Statement s = null;
		ResultSet rs = null;
		
		// Prepeare query
		String sql = "SELECT * FROM ServiceRegistry WHERE ";
		sql += "Name LIKE '" + getDBFormat(name) + "'";
		sql += " AND Type LIKE '" + getDBFormat(type) + "'";
		sql += " AND Location LIKE '" + getDBFormat(location) + "'";
		sql += " AND Owner LIKE '" + getDBFormat(owner) + "'";
		sql += " AND Description LIKE '" + getDBFormat(description) + "'";
		
		// Execute query
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			s = sqlConn.createStatement();
			s.executeQuery(sql);
			rs = s.getResultSet();
			while (rs.next()) {
				result += "  <Service ";
				result += "ID=\"" + rs.getInt("ID") +"\" ";
				result += "ServiceUri=\"" + xmlEscape(rs.getString("ServiceURI")) +"\" ";
				result += "DescriptionUri=\"" + xmlEscape(rs.getString("DescriptionURI")) +"\" ";
				result += "Name=\"" + xmlEscape(rs.getString("Name")) +"\" ";
				result += "Type=\"" + xmlEscape(rs.getString("Type")) +"\" ";
				result += "Location=\"" + xmlEscape(rs.getString("Location")) +"\" ";
				result += "Owner=\"" + xmlEscape(rs.getString("Owner")) +"\" ";
				result += "Description=\"" + xmlEscape(rs.getString("Description")) +"\" />";
				result += newline;
			}
			rs.close();
			s.close();
			sqlConn.close();
		} 
		catch (SQLException e) {
			Log log = Log.getInstance();
			log.writeToLog("Unabe to read service list from database. Message:"
					+ e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		
		return result + " </Servicelist>";
	}
	
	
	
	/**
	 * Gets all registered information about the service specified by the provided
	 * service URI.
	 * 
	 * @param serviceUri The Uri to the service. This location is required to be unique 
	 *         in registry and this ensures that only one service is returned.
	 *            
	 * @return A String with a all info about the requested service in XML-format. The 
	 *         format used is the minimum for a well-formed XML-document, containing only 
	 *         one element formed like this:<br/>
	 *         &lt;Service ID='' ServiceUri='' DescriptionUri='' Name='' 
	 *         Type='' Location='' Owner='' /&gt;
	 */
	public String getServiceDetails(String serviceUri) {
		String sql = "SELECT * FROM ServiceRegistry WHERE serviceURI LIKE '" + serviceUri + "'";
		return getServiceInfo(sql);
	}

	
	
	/**
	 * Gets all registered information about the service specified by the provided
	 * service ID.
	 * 
	 * @param serviceId The unique id refrencing the service.
	 *            
	 * @return A String with a all info about the requested service in XML-format. The 
	 *         format used is the minimum for a well-formed XML-document, containing only 
	 *         one element formed like this:<br/>
	 *         &lt;Service ID='' ServiceUri='' DescriptionUri='' Name='' 
	 *         Type='' Location='' Owner='' /&gt;
	 */
	public String getServiceDetails(int serviceId) {
		String sql = "SELECT * FROM ServiceRegistry WHERE ID = " + serviceId;
		return getServiceInfo(sql);
	}
	
	
	
	
	/**
	 * Get the details for the services which matches the query parameter.
	 * 
	 * @param sql the sql string to use in the query
	 * 
	 * @return An XML-string string containg the query result if a matches is
	 * found, "NO SERVICES MATCHES THE QUERY" otherwise.
	 */
	private String getServiceInfo(String sql) {
		//Create some variables 
		String result = "NO SERVICES MATCHES THE QUERY";
		Statement s = null;
		ResultSet rs = null;
		
		// Execute query
		try {
			if (sqlConn.isClosed()) {
				sqlConn = connectToDatabase();
			}
			s = sqlConn.createStatement();
			s.executeQuery(sql);
			rs = s.getResultSet();
			if (rs.next()) {
				result = "<Service ";
				result += "ID=\"" + rs.getInt("ID") +"\" ";
				result += "ServiceUri=\"" + xmlEscape(rs.getString("ServiceURI")) +"\" ";
				result += "DescriptionUri=\"" + xmlEscape(rs.getString("DescriptionURI")) +"\" ";
				result += "Name=\"" + xmlEscape(rs.getString("Name")) +"\" ";
				result += "Type=\"" + xmlEscape(rs.getString("Type")) +"\" ";
				result += "Location=\"" + xmlEscape(rs.getString("Location")) +"\" ";
				result += "Owner=\"" + xmlEscape(rs.getString("Owner")) +"\" ";
				result += "Description=\"" + xmlEscape(rs.getString("Description")) +"\" />";
			}
			rs.close();
			s.close();
			sqlConn.close();
		} 
		catch (SQLException e) {
			Log log = Log.getInstance();
			log.writeToLog("Unable to read service information from database. Message:"
					+ e.getMessage(), Log.ERROR);
			e.printStackTrace();
		}
		
		//Return result
		return result;
	}
	
	
	
    /**
     * Replace special characters with XML escapes. 
     * XML has the following five predeclared entities:
     * <pre>
     * &amp; <small>(ampersand)</small> is replaced by &amp;amp;
     * &lt; <small>(less than)</small> is replaced by &amp;lt;
     * &gt; <small>(greater than)</small> is replaced by &amp;gt;
     * &apos; <small>(apostrophe)</small> is replaced by &amp;apos;
     * &quot; <small>(double quote)</small> is replaced by &amp;quot;
     * </pre>
     * @param string The string to be escaped.
     * @return The escaped string.
     */
    private String xmlEscape(String string) {
        if (string == null)
        	string = "";
    	StringBuffer sb = new StringBuffer();
        for (int i = 0, len = string.length(); i < len; i++) {
            char c = string.charAt(i);
            switch (c) {
            case '&':
                sb.append("&amp;");
                break;
            case '<':
                sb.append("&lt;");
                break;
            case '>':
                sb.append("&gt;");
                break;
            case '\'':
                sb.append("&apos;");
                break;
            case '"':
                sb.append("&quot;");
                break;
            default:
                sb.append(c);
            }
        }
        return sb.toString();
    }
	
	
    
	/**
	 * Gets the DB format needed to return all results matching the contents of
	 * the provided string. If a null-string or a empty string is presented,
	 * a string which matches all records is returned.
	 * 
	 * @param queryString the unformatted straing
	 * 
	 * @return the DB format
	 */
	private String getDBFormat(String queryString) {
		if (queryString == null)
			return "%";
		else if (queryString.length() == 0)
			return "%";
		else
			return "%" + queryString + "%";
	}
	
	
	
	/**
	 * Gets the DB insert format for the provided string. Simple check that
	 * will replace null value with an empty string.
	 * 
	 * @param insertString the insert string
	 * 
	 * @return the DB insert formated string
	 */
	private String getDBInsertFormat(String insertString) {
		if (insertString == null)
			return "";
		else 
			return insertString.replace("'", "&apos;");
	}
	
}
