/*
 * UbiCollab Service Registry is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is distributable under the Apache License v2.0 (http://www.apache.org/licenses/).
 * 
 * For more information, please contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import no.ubicollab.osgi.service.registry.ServiceRegistry;

/**
 * An interface for UbiCollab Service Registry. Provides the neccessary functionality 
 * for the centraliced service registry. The registry will be accessible through web 
 * service calls.
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class ServiceRegistryImpl implements ServiceRegistry {

	/** The database connector to use for queries. */
	DBConnection dbConn = null;
	
	
	
	/**
	 * This Constructor creates a new instance.
	 * 
	 * @param dbConn the db connection to use for queries
	 */
	public ServiceRegistryImpl(DBConnection dbConn) {
		this.dbConn = dbConn;
	}
	
	
	
	/**
	 * Adds a service reference to the registry. Use this method to add new
	 * services which can be found by other users.
	 * 
	 * @param friendlyName A selected name the user should be able to understand
	 * @param type Classification of service (in service type hierarchy)
	 * @param descriptionURI The description URI for the service
	 * @param owner The owner or responsible for the service
	 * @param description a textual service description
	 * @param location The location of the service
	 * @param serviceURI The Url to the service bundle.
	 * 
	 * @return <code>true</code> if the service refrence is successfully added,
	 *         <code>false</code> otherwise.
	 */
	public boolean addService(String serviceURI, String descriptionURI, String friendlyName, String type, String location, String owner, String description) {
		return dbConn.addService(serviceURI, descriptionURI, friendlyName, type, location, owner, description);
	}
	
	
	
	/**
	 * Gets all registered information about the service specified by the provided
	 * <code>serviceUri</code>.
	 * <p>
	 * The format used for the returned data is the minimum for a well-formed 
	 * XML-document, containing only one element with this syntax:<br/>
	 * <pre>
	 * &lt;Service ID=&quot;int&quot; 
	 *   ServiceUri=&quot;String&quot; 
	 *   DescriptionUri=&quot;String&quot; 
	 *   Name=&quot;String&quot; 
	 *   Type=&quot;String&quot; 
	 *   Location=&quot;String&quot;
	 *   Owner=&quot;String&quot; 
	 *   Description=&quot;String&quot;/&gt;
	 * </pre>     
	 * @param serviceUri The service URI that uniquly identifies the service.
	 *            
	 * @return A <code>String</code> with an XML-document if successful,
	 *         &quot;NO SERVICES MATCHES THE QUERY&quot; otherwise. 
	 */
	public String getServiceDetails(String serviceUri) {
		return dbConn.getServiceDetails(serviceUri);
	}
	
	
	
	/**
	 * Gets all registered information about the service specified by the provided
	 * <code>serviceId</code>.
	 * <p>
	 * The format used for the returned data is the minimum for a well-formed 
	 * XML-document, containing only one element with this syntax:<br/>
	 * <pre>
	 * &lt;Service ID=&quot;int&quot; 
	 *   ServiceUri=&quot;String&quot; 
	 *   DescriptionUri=&quot;String&quot; 
	 *   Name=&quot;String&quot; 
	 *   Type=&quot;String&quot; 
	 *   Location=&quot;String&quot;
	 *   Owner=&quot;String&quot; 
	 *   Description=&quot;String&quot;/&gt;<br/>
	 * </pre>     
	 * @param serviceId The unique id refrencing the service.
	 *            
	 * @return A <code>String</code> with an XML-document if successful,
	 *         &quot;NO SERVICES MATCHES THE QUERY&quot; otherwise. 
	 */
	public String getServiceDetails(int serviceId) {
		return dbConn.getServiceDetails(serviceId);
	}
	
	
	
	/**
	 * This method returns an XML-formatted list containing all the services 
	 * in the registry. 
	 * <p>
	 * The xml-structure used to represent the returned service list is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Servicelist&gt;
	 *          &lt;Service ID=&quot;int&quot; 
	 *            ServiceUri=&quot;String&quot; 
	 *            DescriptionUri=&quot;String&quot; 
	 *            Name=&quot;String&quot; 
	 *            Type=&quot;String&quot; 
	 *            Location=&quot;String&quot;
	 *            Owner=&quot;String&quot; 
	 *            Description=&quot;String&quot;/&gt;<br/> 
	 *          ... more services ...<br/>
	 *      &lt;/Servicelist&gt;
	 * </pre>
	 * 
	 * @return A <code>String</code> containing an XML-formatted list with 0..n 
	 *         service elements.
	 */
	public String getServices() {
		return dbConn.getServiceList("", "", "", "", "");
	}
	
	
	
	/**
	 * Method to return a list of service refrences matching the criteria specified in
	 * the query. The different search criteria can be freely combined and also
	 * partial hits will be returned. Criterias not used should contain an empty string.
	 * <p>
	 * The xml-structure used to represent the returned service list is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Servicelist&gt;
	 *          &lt;Service ID=&quot;int&quot; 
	 *            ServiceUri=&quot;String&quot; 
	 *            DescriptionUri=&quot;String&quot; 
	 *            Name=&quot;String&quot; 
	 *            Type=&quot;String&quot; 
	 *            Location=&quot;String&quot;
	 *            Owner=&quot;String&quot; 
	 *            Description=&quot;String&quot;/&gt;<br/> 
	 *          ... more services ...<br/>
	 *      &lt;/Servicelist&gt;
	 * </pre>
	 * 
	 * @param type the type of service, or a partial type specification
	 * @param owner the owner of the service, or a part of the owners name
	 * @param description the textual description of the service
	 * @param location the location for the service, or a part of the location
	 * @param name the name of the service, or a part of it
	 * 
	 * @return A <code>String</code> containing an XML-formatted list with 0..n 
	 *         service elements.
	 */
	public String getServices(String name, String type, String location, String owner, String description) {
		return dbConn.getServiceList(name, type, location, owner, description);
	}
	
	
	
	/**
	 * Removes the service which matches the provided service ID.
	 * 
	 * @param serviceId The service ID for the service to remove
	 * 
	 * @return <code>true</code> if the service reference is successfully removed, 
	 *         <code>false</code> otherwise.
	 */
	public boolean deleteService(int serviceId) {
		return dbConn.deleteService(serviceId);
	}
	
	
	
	/**
	 * Read from the log and return selected log items in a XML-formatted string.
	 * <p>
	 * The xml-structure used to represent the returned log is:<br/>
	 * <pre>
	 *   &lt;?xml version='1.0'?&gt;
	 *      &lt;Log&gt;
	 *          &lt;Logitem Date=&quot;dd.mm.yyyy&quot; 
	 *            Time=&quot;hh:mm:ss&quot; 
	 *            Level=&quot;String&quot; 
	 *            Message=&quot;String&quot;/&gt;<br/> 
	 *          ... more log items ...<br/>
	 *      &lt;/Log&gt;
	 * </pre>
	 * 
	 * @param logLevel The log level to include in the report. Valid arguments are:
	 *      'DEBUG', 'INFO', 'WARNING', 'ERROR', 'FATAL' and 'ALL' for all types.
	 * @param numItems The number of log items to include in the report
	 * 
	 * @return the <code>String</code> containg the resulting XML
	 */
	public String getLogItems(String logLevel, int numItems) {
		Log log = Log.getInstance();
		return log.readFromLog(logLevel, numItems);
	}


	/**
	 * Updates the properties of service reference in the service registry. The
	 * key used to select which service to update is <code>serviceID</code>.
	 * 
	 * @param serviceID the service ID for the service to update
	 * @param friendlyName A selected name the user should be able to understand
	 * @param descriptionURI The description URI for the service
	 * @param type Classification of service (in service type hierarchy)
	 * @param owner The owner or responsible for the service
	 * @param description a textual service description
	 * @param location The location of the service
	 * @param serviceURI The Url to the service bundle.
	 * 
	 * @return <code>true</code> if the service refrence is successfully updated,
	 *         <code>false</code> otherwise.
	 */
	public boolean updateService(int serviceID, String serviceURI, String descriptionURI, String friendlyName, String type, String location, String owner, String description) {
		return dbConn.updateService(serviceID, serviceURI, descriptionURI, friendlyName, type, location, owner, description);
	}
	
}