using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using X10Client.X10Proxy;


namespace X10Client
{   

    /// <summary>
    /// This class provides a GUI for the UbiCollab X10 Proxy service.
    /// </summary>
    public partial class Form1 : Form
    {
        //The implemented proxy service
        private X10ProxyImplService x10ProxyService;

        //The persistant list of X10 devices
        private List<X10Device> x10DeviceList;

        //True if connected, false otherwise
        private bool connected = false;

        private string webServiceString = "/axis/services/X10Proxy?wsdl";

        /// <summary>
        /// This is the constructor for the primary GUI component, the basic form.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            txtUbiNode.Text = "http://localhost:8080";
            cboCOMPort.Text = "COM6";
            x10ProxyService = new X10ProxyImplService();
            LoadDevices();
        }


        /// <summary>
        /// This method will load the persistant list of X10 devices
        /// and populate the device list listview control.
        /// </summary>
        private void LoadDevices()
        {
            x10DeviceList = DeviceStore.LoadDevices();

            this.lwDeviceList.BeginUpdate();
            this.lwDeviceList.Items.Clear();

            foreach (X10Device d in x10DeviceList)
            {
                ListViewItem lvi = new ListViewItem(
                    new string[] { d.FriendlyName, d.House, d.Unit+"" });
                lvi.Tag = d;
                this.lwDeviceList.Items.Add(lvi);
            }

            this.lwDeviceList.EndUpdate();
        }



        /// <summary>
        /// Handler for the turn of device button click
        /// </summary>
        private void btnOff_Click(object sender, EventArgs e)
        {
            if (lwDeviceList.FocusedItem != null && connected)
            {
                X10Device d = (X10Device) lwDeviceList.FocusedItem.Tag;
                x10ProxyService.sendCommand(d.House, d.Unit, false);
            }
            else
                MessageBox.Show("Please select a device prior to issuing commands.");
        }



        /// <summary>
        /// Handler for the turn on device button click
        /// </summary>
        private void btnOn_Click(object sender, EventArgs e)
        {
            if (lwDeviceList.FocusedItem != null && connected)
            {
                X10Device d = (X10Device)lwDeviceList.FocusedItem.Tag;
                x10ProxyService.sendCommand(d.House, d.Unit, true);
            }
            else
                MessageBox.Show("Please select a device prior to issuing commands.");
        }

        
        
        /// <summary>
        /// Method to disconnect from the X10 serial controller.
        /// </summary>
        private void btnCloseConnection_Click(object sender, EventArgs e)
        {
            if (connected)
            {
                x10ProxyService.stopSerialConnection();
                txtUbiNode.Enabled = true;
                btnConnect.Enabled = true;
                cboCOMPort.Enabled = true;
                btnCloseConnection.Enabled = false;
                grpController.Enabled = false;
                connected = false;
            }
           
        }


        
        /// <summary>
        /// Add a new device to the device list.
        /// </summary>
        private void btnAdd_Click(object sender, EventArgs e)
        {
            x10DeviceList.Add(new X10Device(txtName.Text, txtHouse.Text, Int32.Parse(txtUnit.Text)));
            DeviceStore.SaveDevices(x10DeviceList);
            LoadDevices();
        }


        
        /// <summary>
        /// Send command to start a serial conection to an X10 controller on a given COM port.
        /// </summary>
        private void btnConnect_Click(object sender, EventArgs e)
        {
            bool result = true;
            try
            {
                x10ProxyService.Url = txtUbiNode.Text + webServiceString;
                //result = x10ProxyService.startSerialConnection(cboCOMPort.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                result = false;
                return;
            }
            
            if (result)
            {
                txtUbiNode.Enabled = false;
                btnConnect.Enabled = false;
                cboCOMPort.Enabled = false;
                grpController.Enabled = true;
                btnCloseConnection.Enabled = true;
                connected = true;
            }
            else
                MessageBox.Show("Connection failed.");
        }



        /// <summary>
        /// Delete a X10 device from the device list and from persistant storage.
        /// </summary>
        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (lwDeviceList.FocusedItem != null && connected)
            {
                X10Device deleteD = (X10Device)lwDeviceList.FocusedItem.Tag;
                foreach (X10Device d in x10DeviceList)
                    if (deleteD.FriendlyName == d.FriendlyName)
                        x10DeviceList.Remove(d);
                DeviceStore.SaveDevices(x10DeviceList);
                LoadDevices();
            }
            else
                MessageBox.Show("You must select a device to delete it.");
        }


    }
}