package org.ubicollab.X10Proxy;

import java.io.IOException;
import com.jpeterson.x10.Transmitter;
import com.jpeterson.x10.event.AddressEvent;
import com.jpeterson.x10.event.DimEvent;
import com.jpeterson.x10.event.BrightEvent;
import com.jpeterson.x10.event.OffEvent;
import com.jpeterson.x10.event.OnEvent;
import com.jpeterson.x10.event.X10Event;
import com.jpeterson.x10.Gateway;
import com.jpeterson.x10.GatewayException;
import com.jpeterson.x10.SerialGateway;


/**
 * The Class <code>X10Impl</code> implements the methods neccessary to connect
 * to, disconnect and send on/off/dimming/brighten commands with an CM11A 
 * serial RS232 X10 controller.
 * 
 * TODO: More x10 functions should be implemented here.
 * 
 * @author Kim-Steve Johansen
 * @version 0.1
 */
public class X10Impl {

	/** The transmitter. */
	private Transmitter transmitter;
	
	/** The start result. */
	private boolean startResult = false;
	
	
	/**
	 * The Constructor.
	 * 
	 * @param port the port
	 */
	public X10Impl(String port) {
		try {
			transmitter = (Transmitter) Class.forName(
					"com.jpeterson.x10.module.CM11A").newInstance();
		} catch (ClassNotFoundException e) {
			System.err.println("Unknown class: com.jpeterson.x10.module.CM11A");
			e.printStackTrace();
			startResult = false;
		} catch (InstantiationException e) {
			System.err
					.println("Error instantiating com.jpeterson.x10.module.CM11A");
			e.printStackTrace();
			startResult = false;
		} catch (IllegalAccessException e) {
			System.err
					.println("Illegally accessing com.jpeterson.x10.module.CM11A");
			e.printStackTrace();
			startResult = false;
		}

		Gateway gateway = (Gateway) transmitter;
		if ((transmitter instanceof SerialGateway) && (port != null)) {
			((SerialGateway) transmitter).setPortName(port);
		}
		try {
			gateway.allocate();
		} catch (Exception e) {
			startResult = false;
			e.printStackTrace();
		}
	}
	
	
    /**
     * This method sends dim or brighten events to the given
     * X10 device.
     * 
     * @param cmnd Either 0 for dim or 1 for brighten
     * @param unit the unit
     * @param house the house
     * @param percentage percentage to change between 1 and 100.
     */
    public void dimDevice(char house, int unit, int cmnd, int percentage) {
        X10Event[] events = null;
        Double dUnits = new Double(percentage * .22);
        int iUnits = dUnits.intValue();
        switch (cmnd)
        {
            case 0:
                events = new X10Event[2];
                events[0] = new AddressEvent(this, house, unit);
                events[1] = new DimEvent(this, house, iUnits, 22);
                break;
            case 1:
                events = new X10Event[2];
                events[0] = new AddressEvent(this, house, unit);
                events[1] = new BrightEvent(this, house, iUnits, 22);
                break;
        }
        if (events != null)
        {
    		for (int i = 0; i < events.length; i++) {
    			try {
    				transmitter.transmit(events[i]);
    			} 
    			catch (IOException e) {
    				e.printStackTrace();
    			}
    		}
        }
    }

	
	
	
	/**
	 * Turn on a device.
	 * 
	 * @param unit the unit to send comand to
	 * @param house the house to send command to
	 */
	public void turnOnDevice(char house, int unit) {
		X10Event[] events = new X10Event[2];
		events[0] = new AddressEvent(this, house, unit);
		events[1] = new OnEvent(this, house);
		for (int i = 0; i < events.length; i++) {
			try {
				transmitter.transmit(events[i]);
			} 
			catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	
	/**
	 * Turn off device.
	 * 
	 * @param unit the unit
	 * @param house the house
	 */
	public void turnOffDevice(char house, int unit) {
		X10Event[] events = new X10Event[2];
		events[0] = new AddressEvent(this, house, unit);
		events[1] = new OffEvent(this, house);
		for (int i = 0; i < events.length; i++) {
			try {
				transmitter.transmit(events[i]);
			} 
			catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	
	/**
	 * Close the serial connection and release resources used.
	 */
	public void close() {
		Gateway gateway = (Gateway) transmitter;

		try {
			gateway.waitGatewayState(Transmitter.QUEUE_EMPTY);
			System.out.println("Transmitted all pending X10 commands.");
		} 
		catch (InterruptedException e) {
			e.printStackTrace();
		}

		try {
			gateway.deallocate();
			System.out.println("X10 gateway has released resources.");
		} 
		catch (GatewayException e) {
			e.printStackTrace();
		}
		transmitter = null;
	}


	/**
	 * Checks if start result is ok.
	 * 
	 * @return true, if start succeded or false othervise
	 */
	public boolean isStarted() {
		return startResult;
	}

}